declare module "windows.networking.sockets" {
  export class RoundTripTimeStatistics {
    variance: Number;
    max: Number;
    min: Number;
    sum: Number;
    constructor();
  }

  export class BandwidthStatistics {
    outboundBitsPerSecond: Number;
    inboundBitsPerSecond: Number;
    outboundBitsPerSecondInstability: Number;
    inboundBitsPerSecondInstability: Number;
    outboundBandwidthPeaked: Boolean;
    inboundBandwidthPeaked: Boolean;
    constructor();
  }

  export enum ControlChannelTriggerStatus {
    hardwareSlotRequested,
    softwareSlotAllocated,
    hardwareSlotAllocated,
    policyError,
    systemError,
    transportDisconnected,
    serviceUnavailable,
  }

  export enum ControlChannelTriggerResourceType {
    requestSoftwareSlot,
    requestHardwareSlot,
  }

  export enum ControlChannelTriggerResetReason {
    fastUserSwitched,
    lowPowerExit,
    quietHoursExit,
    applicationRestart,
  }

  export enum SocketMessageType {
    binary,
    utf8,
  }

  export enum SocketProtectionLevel {
    plainSocket,
    ssl,
    sslAllowNullEncryption,
    bluetoothEncryptionAllowNullAuthentication,
    bluetoothEncryptionWithAuthentication,
    ssl3AllowWeakEncryption,
    tls10,
    tls11,
    tls12,
  }

  export enum SocketQualityOfService {
    normal,
    lowLatency,
  }

  export enum SocketErrorStatus {
    unknown,
    operationAborted,
    httpInvalidServerResponse,
    connectionTimedOut,
    addressFamilyNotSupported,
    socketTypeNotSupported,
    hostNotFound,
    noDataRecordOfRequestedType,
    nonAuthoritativeHostNotFound,
    classTypeNotFound,
    addressAlreadyInUse,
    cannotAssignRequestedAddress,
    connectionRefused,
    networkIsUnreachable,
    unreachableHost,
    networkIsDown,
    networkDroppedConnectionOnReset,
    softwareCausedConnectionAbort,
    connectionResetByPeer,
    hostIsDown,
    noAddressesFound,
    tooManyOpenFiles,
    messageTooLong,
    certificateExpired,
    certificateUntrustedRoot,
    certificateCommonNameIsIncorrect,
    certificateWrongUsage,
    certificateRevoked,
    certificateNoRevocationCheck,
    certificateRevocationServerOffline,
    certificateIsInvalid,
  }

  export enum SocketSslErrorSeverity {
    none,
    ignorable,
    fatal,
  }

  export class ControlChannelTrigger {
    serverKeepAliveIntervalInMinutes: Number;
    controlChannelTriggerId: String;
    currentKeepAliveIntervalInMinutes: Number;
    keepAliveTrigger: Object;
    pushNotificationTrigger: Object;
    transportObject: Object;
    constructor();
    constructor(channelId: String, serverKeepAliveIntervalInMinutes: Number);
    constructor(channelId: String, serverKeepAliveIntervalInMinutes: Number, resourceRequestType: ControlChannelTriggerResourceType);

    usingTransport(transport: Object): void;

    waitForPushEnabled(): ControlChannelTriggerStatus;

    decreaseNetworkKeepAliveInterval(): void;

    flushTransport(): void;

    close(): void;
  }

  export class IControlChannelTriggerEventDetails {
    controlChannelTrigger: ControlChannelTrigger;
    constructor();

  }

  export class IControlChannelTriggerResetEventDetails {
    hardwareSlotReset: Boolean;
    resetReason: ControlChannelTriggerResetReason;
    softwareSlotReset: Boolean;
    constructor();

  }

  export class DatagramSocketControl {
    inboundBufferSizeInBytes: Number;
    dontFragment: Boolean;
    qualityOfService: SocketQualityOfService;
    outboundUnicastHopLimit: Number;
    constructor();

  }

  export class DatagramSocketInformation {
    localAddress: Object;
    localPort: String;
    remoteAddress: Object;
    remotePort: String;
    constructor();

  }

  export class DatagramSocket {
    control: DatagramSocketControl;
    information: DatagramSocketInformation;
    outputStream: Object;
    constructor();

    static getEndpointPairsAsync(remoteHostName: Object, remoteServiceName: String, callback: (error: Error, result: Object) => void): void ;
    static getEndpointPairsAsync(remoteHostName: Object, remoteServiceName: String, sortOptions: Number, callback: (error: Error, result: Object) => void): void ;


    connectAsync(remoteHostName: Object, remoteServiceName: String, callback: (error: Error) => void): void ;
    connectAsync(endpointPair: Object, callback: (error: Error) => void): void ;

    bindServiceNameAsync(localServiceName: String, callback: (error: Error) => void): void ;
    bindServiceNameAsync(localServiceName: String, adapter: Object, callback: (error: Error) => void): void ;

    bindEndpointAsync(localHostName: Object, localServiceName: String, callback: (error: Error) => void): void ;

    getOutputStreamAsync(remoteHostName: Object, remoteServiceName: String, callback: (error: Error, result: Object) => void): void ;
    getOutputStreamAsync(endpointPair: Object, callback: (error: Error, result: Object) => void): void ;

    joinMulticastGroup(host: Object): void;

    close(): void;
    addListener(type: "MessageReceived", listener: (ev: Event) => void): void ;
    removeListener(type: "MessageReceived", listener: (ev: Event) => void): void ;
    on(type: "MessageReceived", listener: (ev: Event) => void): void ;
    off(type: "MessageReceived", listener: (ev: Event) => void): void ;
    
    addListener(type: string, listener: (ev: Event) => void): void ;
    removeListener(type: string, listener: (ev: Event) => void): void ;
    on(type: string, listener: (ev: Event) => void): void ;
    off(type: string, listener: (ev: Event) => void): void ;
    

  }

  export class DatagramSocketMessageReceivedEventArgs {
    localAddress: Object;
    remoteAddress: Object;
    remotePort: String;
    constructor();

    getDataReader(): Object;

    getDataStream(): Object;

  }

  export class StreamSocketControl {
    qualityOfService: SocketQualityOfService;
    outboundUnicastHopLimit: Number;
    outboundBufferSizeInBytes: Number;
    noDelay: Boolean;
    keepAlive: Boolean;
    ignorableServerCertificateErrors: Object;
    constructor();

  }

  export class StreamSocketInformation {
    bandwidthStatistics: BandwidthStatistics;
    localAddress: Object;
    localPort: String;
    protectionLevel: SocketProtectionLevel;
    remoteAddress: Object;
    remoteHostName: Object;
    remotePort: String;
    remoteServiceName: String;
    roundTripTimeStatistics: RoundTripTimeStatistics;
    sessionKey: Object;
    serverCertificate: Object;
    serverCertificateErrorSeverity: SocketSslErrorSeverity;
    serverCertificateErrors: Object;
    serverIntermediateCertificates: Object;
    constructor();

  }

  export class StreamSocket {
    control: StreamSocketControl;
    information: StreamSocketInformation;
    inputStream: Object;
    outputStream: Object;
    constructor();

    connectAsync(endpointPair: Object, callback: (error: Error) => void): void ;
    connectAsync(remoteHostName: Object, remoteServiceName: String, callback: (error: Error) => void): void ;
    connectAsync(endpointPair: Object, protectionLevel: SocketProtectionLevel, callback: (error: Error) => void): void ;
    connectAsync(remoteHostName: Object, remoteServiceName: String, protectionLevel: SocketProtectionLevel, callback: (error: Error) => void): void ;
    connectAsync(remoteHostName: Object, remoteServiceName: String, protectionLevel: SocketProtectionLevel, adapter: Object, callback: (error: Error) => void): void ;

    upgradeToSslAsync(protectionLevel: SocketProtectionLevel, validationHostName: Object, callback: (error: Error) => void): void ;

    close(): void;
  }

  export class StreamSocketListenerControl {
    qualityOfService: SocketQualityOfService;
    constructor();

  }

  export class StreamSocketListenerInformation {
    localPort: String;
    constructor();

  }

  export class StreamSocketListener {
    control: StreamSocketListenerControl;
    information: StreamSocketListenerInformation;
    constructor();

    bindServiceNameAsync(localServiceName: String, callback: (error: Error) => void): void ;
    bindServiceNameAsync(localServiceName: String, protectionLevel: SocketProtectionLevel, callback: (error: Error) => void): void ;
    bindServiceNameAsync(localServiceName: String, protectionLevel: SocketProtectionLevel, adapter: Object, callback: (error: Error) => void): void ;

    bindEndpointAsync(localHostName: Object, localServiceName: String, callback: (error: Error) => void): void ;

    close(): void;
    addListener(type: "ConnectionReceived", listener: (ev: Event) => void): void ;
    removeListener(type: "ConnectionReceived", listener: (ev: Event) => void): void ;
    on(type: "ConnectionReceived", listener: (ev: Event) => void): void ;
    off(type: "ConnectionReceived", listener: (ev: Event) => void): void ;
    
    addListener(type: string, listener: (ev: Event) => void): void ;
    removeListener(type: string, listener: (ev: Event) => void): void ;
    on(type: string, listener: (ev: Event) => void): void ;
    off(type: string, listener: (ev: Event) => void): void ;
    

  }

  export class StreamSocketListenerConnectionReceivedEventArgs {
    socket: StreamSocket;
    constructor();

  }

  export class IWebSocketControl {
    outboundBufferSizeInBytes: Number;
    proxyCredential: Object;
    serverCredential: Object;
    supportedProtocols: Object;
    constructor();

  }

  export class IWebSocketInformation {
    bandwidthStatistics: BandwidthStatistics;
    localAddress: Object;
    protocol: String;
    constructor();

  }

  export class IWebSocket {
    outputStream: Object;
    constructor();

    connectAsync(uri: Object, callback: (error: Error) => void): void ;

    setRequestHeader(headerName: String, headerValue: String): void;

    addListener(type: "Closed", listener: (ev: Event) => void): void ;
    removeListener(type: "Closed", listener: (ev: Event) => void): void ;
    on(type: "Closed", listener: (ev: Event) => void): void ;
    off(type: "Closed", listener: (ev: Event) => void): void ;
    
    addListener(type: string, listener: (ev: Event) => void): void ;
    removeListener(type: string, listener: (ev: Event) => void): void ;
    on(type: string, listener: (ev: Event) => void): void ;
    off(type: string, listener: (ev: Event) => void): void ;
    

  }

  export class WebSocketClosedEventArgs {
    code: Number;
    reason: String;
    constructor();

  }

  export class MessageWebSocketControl {
    messageType: SocketMessageType;
    maxMessageSize: Number;
    serverCredential: Object;
    proxyCredential: Object;
    outboundBufferSizeInBytes: Number;
    supportedProtocols: Object;
    constructor();

  }

  export class MessageWebSocketInformation {
    bandwidthStatistics: BandwidthStatistics;
    localAddress: Object;
    protocol: String;
    constructor();

  }

  export class MessageWebSocket {
    control: MessageWebSocketControl;
    information: MessageWebSocketInformation;
    outputStream: Object;
    constructor();

    connectAsync(uri: Object, callback: (error: Error) => void): void ;

    setRequestHeader(headerName: String, headerValue: String): void;

    close(): void;
    addListener(type: "MessageReceived", listener: (ev: Event) => void): void ;
    removeListener(type: "MessageReceived", listener: (ev: Event) => void): void ;
    on(type: "MessageReceived", listener: (ev: Event) => void): void ;
    off(type: "MessageReceived", listener: (ev: Event) => void): void ;
    
    addListener(type: "Closed", listener: (ev: Event) => void): void ;
    removeListener(type: "Closed", listener: (ev: Event) => void): void ;
    on(type: "Closed", listener: (ev: Event) => void): void ;
    off(type: "Closed", listener: (ev: Event) => void): void ;
    
    addListener(type: string, listener: (ev: Event) => void): void ;
    removeListener(type: string, listener: (ev: Event) => void): void ;
    on(type: string, listener: (ev: Event) => void): void ;
    off(type: string, listener: (ev: Event) => void): void ;
    

  }

  export class MessageWebSocketMessageReceivedEventArgs {
    messageType: SocketMessageType;
    constructor();

    getDataReader(): Object;

    getDataStream(): Object;

  }

  export class StreamWebSocketControl {
    noDelay: Boolean;
    serverCredential: Object;
    proxyCredential: Object;
    outboundBufferSizeInBytes: Number;
    supportedProtocols: Object;
    constructor();

  }

  export class StreamWebSocketInformation {
    bandwidthStatistics: BandwidthStatistics;
    localAddress: Object;
    protocol: String;
    constructor();

  }

  export class StreamWebSocket {
    control: StreamWebSocketControl;
    information: StreamWebSocketInformation;
    inputStream: Object;
    outputStream: Object;
    constructor();

    connectAsync(uri: Object, callback: (error: Error) => void): void ;

    setRequestHeader(headerName: String, headerValue: String): void;

    close(): void;
    addListener(type: "Closed", listener: (ev: Event) => void): void ;
    removeListener(type: "Closed", listener: (ev: Event) => void): void ;
    on(type: "Closed", listener: (ev: Event) => void): void ;
    off(type: "Closed", listener: (ev: Event) => void): void ;
    
    addListener(type: string, listener: (ev: Event) => void): void ;
    removeListener(type: string, listener: (ev: Event) => void): void ;
    on(type: string, listener: (ev: Event) => void): void ;
    off(type: string, listener: (ev: Event) => void): void ;
    

  }

  export class WebSocketKeepAlive {
    constructor();

    run(taskInstance: Object): void;

  }

  export class SocketError {
    constructor();

    static getStatus(hresult: Number): SocketErrorStatus;


  }

  export class WebSocketError {
    constructor();

    static getStatus(hresult: Number): Number;


  }

}



