// Copyright (c) Microsoft Corporation
// All rights reserved. 
//
// Licensed under the Apache License, Version 2.0 (the ""License""); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
//
// THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT. 
//
// See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.

#define NTDDI_VERSION 0x06010000

#include <v8.h>
#include <string>
#include <node_object_wrap.h>
#include <ppltasks.h>
#include "CollectionsConverter.h"
#include "CollectionsWrap.h"
#include "node-async.h"
#include "NodeRtUtils.h"
#include "OpaqueWrapper.h"
#include "WrapperBase.h"

#using <Windows.WinMD>

// this undefs fixes the issues of compiling Windows.Data.Json, Windows.Storag.FileProperties, and Windows.Stroage.Search
// Some of the node header files brings windows definitions with the same names as some of the WinRT methods
#undef DocumentProperties
#undef GetObject
#undef CreateEvent
#undef FindText
#undef SendMessage

const char* REGISTRATION_TOKEN_MAP_PROPERTY_NAME = "__registrationTokenMap__";

using namespace v8;
using namespace node;
using namespace concurrency;

namespace NodeRT { namespace Windows { namespace Networking { namespace Sockets { 
  v8::Handle<v8::Value> WrapControlChannelTrigger(::Windows::Networking::Sockets::ControlChannelTrigger^ wintRtInstance);
  ::Windows::Networking::Sockets::ControlChannelTrigger^ UnwrapControlChannelTrigger(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIControlChannelTriggerEventDetails(::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^ wintRtInstance);
  ::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^ UnwrapIControlChannelTriggerEventDetails(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIControlChannelTriggerResetEventDetails(::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^ wintRtInstance);
  ::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^ UnwrapIControlChannelTriggerResetEventDetails(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapDatagramSocketControl(::Windows::Networking::Sockets::DatagramSocketControl^ wintRtInstance);
  ::Windows::Networking::Sockets::DatagramSocketControl^ UnwrapDatagramSocketControl(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapDatagramSocketInformation(::Windows::Networking::Sockets::DatagramSocketInformation^ wintRtInstance);
  ::Windows::Networking::Sockets::DatagramSocketInformation^ UnwrapDatagramSocketInformation(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapDatagramSocket(::Windows::Networking::Sockets::DatagramSocket^ wintRtInstance);
  ::Windows::Networking::Sockets::DatagramSocket^ UnwrapDatagramSocket(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapDatagramSocketMessageReceivedEventArgs(::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^ wintRtInstance);
  ::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^ UnwrapDatagramSocketMessageReceivedEventArgs(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapStreamSocketControl(::Windows::Networking::Sockets::StreamSocketControl^ wintRtInstance);
  ::Windows::Networking::Sockets::StreamSocketControl^ UnwrapStreamSocketControl(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapStreamSocketInformation(::Windows::Networking::Sockets::StreamSocketInformation^ wintRtInstance);
  ::Windows::Networking::Sockets::StreamSocketInformation^ UnwrapStreamSocketInformation(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapStreamSocket(::Windows::Networking::Sockets::StreamSocket^ wintRtInstance);
  ::Windows::Networking::Sockets::StreamSocket^ UnwrapStreamSocket(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapStreamSocketListenerControl(::Windows::Networking::Sockets::StreamSocketListenerControl^ wintRtInstance);
  ::Windows::Networking::Sockets::StreamSocketListenerControl^ UnwrapStreamSocketListenerControl(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapStreamSocketListenerInformation(::Windows::Networking::Sockets::StreamSocketListenerInformation^ wintRtInstance);
  ::Windows::Networking::Sockets::StreamSocketListenerInformation^ UnwrapStreamSocketListenerInformation(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapStreamSocketListener(::Windows::Networking::Sockets::StreamSocketListener^ wintRtInstance);
  ::Windows::Networking::Sockets::StreamSocketListener^ UnwrapStreamSocketListener(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapStreamSocketListenerConnectionReceivedEventArgs(::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^ wintRtInstance);
  ::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^ UnwrapStreamSocketListenerConnectionReceivedEventArgs(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIWebSocketControl(::Windows::Networking::Sockets::IWebSocketControl^ wintRtInstance);
  ::Windows::Networking::Sockets::IWebSocketControl^ UnwrapIWebSocketControl(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIWebSocketInformation(::Windows::Networking::Sockets::IWebSocketInformation^ wintRtInstance);
  ::Windows::Networking::Sockets::IWebSocketInformation^ UnwrapIWebSocketInformation(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIWebSocket(::Windows::Networking::Sockets::IWebSocket^ wintRtInstance);
  ::Windows::Networking::Sockets::IWebSocket^ UnwrapIWebSocket(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapWebSocketClosedEventArgs(::Windows::Networking::Sockets::WebSocketClosedEventArgs^ wintRtInstance);
  ::Windows::Networking::Sockets::WebSocketClosedEventArgs^ UnwrapWebSocketClosedEventArgs(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMessageWebSocketControl(::Windows::Networking::Sockets::MessageWebSocketControl^ wintRtInstance);
  ::Windows::Networking::Sockets::MessageWebSocketControl^ UnwrapMessageWebSocketControl(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMessageWebSocketInformation(::Windows::Networking::Sockets::MessageWebSocketInformation^ wintRtInstance);
  ::Windows::Networking::Sockets::MessageWebSocketInformation^ UnwrapMessageWebSocketInformation(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMessageWebSocket(::Windows::Networking::Sockets::MessageWebSocket^ wintRtInstance);
  ::Windows::Networking::Sockets::MessageWebSocket^ UnwrapMessageWebSocket(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMessageWebSocketMessageReceivedEventArgs(::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^ wintRtInstance);
  ::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^ UnwrapMessageWebSocketMessageReceivedEventArgs(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapStreamWebSocketControl(::Windows::Networking::Sockets::StreamWebSocketControl^ wintRtInstance);
  ::Windows::Networking::Sockets::StreamWebSocketControl^ UnwrapStreamWebSocketControl(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapStreamWebSocketInformation(::Windows::Networking::Sockets::StreamWebSocketInformation^ wintRtInstance);
  ::Windows::Networking::Sockets::StreamWebSocketInformation^ UnwrapStreamWebSocketInformation(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapStreamWebSocket(::Windows::Networking::Sockets::StreamWebSocket^ wintRtInstance);
  ::Windows::Networking::Sockets::StreamWebSocket^ UnwrapStreamWebSocket(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapWebSocketKeepAlive(::Windows::Networking::Sockets::WebSocketKeepAlive^ wintRtInstance);
  ::Windows::Networking::Sockets::WebSocketKeepAlive^ UnwrapWebSocketKeepAlive(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapSocketError(::Windows::Networking::Sockets::SocketError^ wintRtInstance);
  ::Windows::Networking::Sockets::SocketError^ UnwrapSocketError(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapWebSocketError(::Windows::Networking::Sockets::WebSocketError^ wintRtInstance);
  ::Windows::Networking::Sockets::WebSocketError^ UnwrapWebSocketError(Handle<Value> value);
  


  static v8::Handle<v8::Value> InitControlChannelTriggerStatusEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("ControlChannelTriggerStatus"), enumObject);

    enumObject->Set(String::NewSymbol("hardwareSlotRequested"), Integer::New(0));
    enumObject->Set(String::NewSymbol("softwareSlotAllocated"), Integer::New(1));
    enumObject->Set(String::NewSymbol("hardwareSlotAllocated"), Integer::New(2));
    enumObject->Set(String::NewSymbol("policyError"), Integer::New(3));
    enumObject->Set(String::NewSymbol("systemError"), Integer::New(4));
    enumObject->Set(String::NewSymbol("transportDisconnected"), Integer::New(5));
    enumObject->Set(String::NewSymbol("serviceUnavailable"), Integer::New(6));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitControlChannelTriggerResourceTypeEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("ControlChannelTriggerResourceType"), enumObject);

    enumObject->Set(String::NewSymbol("requestSoftwareSlot"), Integer::New(0));
    enumObject->Set(String::NewSymbol("requestHardwareSlot"), Integer::New(1));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitControlChannelTriggerResetReasonEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("ControlChannelTriggerResetReason"), enumObject);

    enumObject->Set(String::NewSymbol("fastUserSwitched"), Integer::New(0));
    enumObject->Set(String::NewSymbol("lowPowerExit"), Integer::New(1));
    enumObject->Set(String::NewSymbol("quietHoursExit"), Integer::New(2));
    enumObject->Set(String::NewSymbol("applicationRestart"), Integer::New(3));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitSocketMessageTypeEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("SocketMessageType"), enumObject);

    enumObject->Set(String::NewSymbol("binary"), Integer::New(0));
    enumObject->Set(String::NewSymbol("utf8"), Integer::New(1));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitSocketProtectionLevelEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("SocketProtectionLevel"), enumObject);

    enumObject->Set(String::NewSymbol("plainSocket"), Integer::New(0));
    enumObject->Set(String::NewSymbol("ssl"), Integer::New(1));
    enumObject->Set(String::NewSymbol("sslAllowNullEncryption"), Integer::New(2));
    enumObject->Set(String::NewSymbol("bluetoothEncryptionAllowNullAuthentication"), Integer::New(3));
    enumObject->Set(String::NewSymbol("bluetoothEncryptionWithAuthentication"), Integer::New(4));
    enumObject->Set(String::NewSymbol("ssl3AllowWeakEncryption"), Integer::New(5));
    enumObject->Set(String::NewSymbol("tls10"), Integer::New(6));
    enumObject->Set(String::NewSymbol("tls11"), Integer::New(7));
    enumObject->Set(String::NewSymbol("tls12"), Integer::New(8));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitSocketQualityOfServiceEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("SocketQualityOfService"), enumObject);

    enumObject->Set(String::NewSymbol("normal"), Integer::New(0));
    enumObject->Set(String::NewSymbol("lowLatency"), Integer::New(1));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitSocketErrorStatusEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("SocketErrorStatus"), enumObject);

    enumObject->Set(String::NewSymbol("unknown"), Integer::New(0));
    enumObject->Set(String::NewSymbol("operationAborted"), Integer::New(1));
    enumObject->Set(String::NewSymbol("httpInvalidServerResponse"), Integer::New(2));
    enumObject->Set(String::NewSymbol("connectionTimedOut"), Integer::New(3));
    enumObject->Set(String::NewSymbol("addressFamilyNotSupported"), Integer::New(4));
    enumObject->Set(String::NewSymbol("socketTypeNotSupported"), Integer::New(5));
    enumObject->Set(String::NewSymbol("hostNotFound"), Integer::New(6));
    enumObject->Set(String::NewSymbol("noDataRecordOfRequestedType"), Integer::New(7));
    enumObject->Set(String::NewSymbol("nonAuthoritativeHostNotFound"), Integer::New(8));
    enumObject->Set(String::NewSymbol("classTypeNotFound"), Integer::New(9));
    enumObject->Set(String::NewSymbol("addressAlreadyInUse"), Integer::New(10));
    enumObject->Set(String::NewSymbol("cannotAssignRequestedAddress"), Integer::New(11));
    enumObject->Set(String::NewSymbol("connectionRefused"), Integer::New(12));
    enumObject->Set(String::NewSymbol("networkIsUnreachable"), Integer::New(13));
    enumObject->Set(String::NewSymbol("unreachableHost"), Integer::New(14));
    enumObject->Set(String::NewSymbol("networkIsDown"), Integer::New(15));
    enumObject->Set(String::NewSymbol("networkDroppedConnectionOnReset"), Integer::New(16));
    enumObject->Set(String::NewSymbol("softwareCausedConnectionAbort"), Integer::New(17));
    enumObject->Set(String::NewSymbol("connectionResetByPeer"), Integer::New(18));
    enumObject->Set(String::NewSymbol("hostIsDown"), Integer::New(19));
    enumObject->Set(String::NewSymbol("noAddressesFound"), Integer::New(20));
    enumObject->Set(String::NewSymbol("tooManyOpenFiles"), Integer::New(21));
    enumObject->Set(String::NewSymbol("messageTooLong"), Integer::New(22));
    enumObject->Set(String::NewSymbol("certificateExpired"), Integer::New(23));
    enumObject->Set(String::NewSymbol("certificateUntrustedRoot"), Integer::New(24));
    enumObject->Set(String::NewSymbol("certificateCommonNameIsIncorrect"), Integer::New(25));
    enumObject->Set(String::NewSymbol("certificateWrongUsage"), Integer::New(26));
    enumObject->Set(String::NewSymbol("certificateRevoked"), Integer::New(27));
    enumObject->Set(String::NewSymbol("certificateNoRevocationCheck"), Integer::New(28));
    enumObject->Set(String::NewSymbol("certificateRevocationServerOffline"), Integer::New(29));
    enumObject->Set(String::NewSymbol("certificateIsInvalid"), Integer::New(30));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitSocketSslErrorSeverityEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("SocketSslErrorSeverity"), enumObject);

    enumObject->Set(String::NewSymbol("none"), Integer::New(0));
    enumObject->Set(String::NewSymbol("ignorable"), Integer::New(1));
    enumObject->Set(String::NewSymbol("fatal"), Integer::New(2));

    return scope.Close(Undefined());
  }



  
  static bool IsRoundTripTimeStatisticsJsObject(Handle<Value> value)
  {
    if (!value->IsObject())
    {
      return false;
    }

    Handle<String> symbol;
    Handle<Object> obj = value.As<Object>();

    symbol = String::NewSymbol("variance");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsUint32())
      {
          return false;
      }
    }
    
    symbol = String::NewSymbol("max");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsUint32())
      {
          return false;
      }
    }
    
    symbol = String::NewSymbol("min");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsUint32())
      {
          return false;
      }
    }
    
    symbol = String::NewSymbol("sum");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsUint32())
      {
          return false;
      }
    }
    
    return true;
  }

  ::Windows::Networking::Sockets::RoundTripTimeStatistics RoundTripTimeStatisticsFromJsObject(Handle<Value> value)
  {
    HandleScope scope;
    ::Windows::Networking::Sockets::RoundTripTimeStatistics returnValue;
    
    if (!value->IsObject())
    {
      ThrowException(Exception::TypeError(NodeRT::Utils::NewString(L"Unexpected type, expected an object")));
      return returnValue;
    }

    Handle<Object> obj = value.As<Object>();
    Handle<String> symbol;

    symbol = String::NewSymbol("variance");
    if (obj->Has(symbol))
    {
      returnValue.Variance = static_cast<unsigned int>(obj->Get(symbol)->IntegerValue());
    }
    
    symbol = String::NewSymbol("max");
    if (obj->Has(symbol))
    {
      returnValue.Max = static_cast<unsigned int>(obj->Get(symbol)->IntegerValue());
    }
    
    symbol = String::NewSymbol("min");
    if (obj->Has(symbol))
    {
      returnValue.Min = static_cast<unsigned int>(obj->Get(symbol)->IntegerValue());
    }
    
    symbol = String::NewSymbol("sum");
    if (obj->Has(symbol))
    {
      returnValue.Sum = static_cast<unsigned int>(obj->Get(symbol)->IntegerValue());
    }
    
    return returnValue;
  }

  Handle<Value> RoundTripTimeStatisticsToJsObject(::Windows::Networking::Sockets::RoundTripTimeStatistics value)
  {
    HandleScope scope;

    Handle<Object> obj = Object::New();

    obj->Set(String::NewSymbol("variance"), Integer::New(value.Variance));
    obj->Set(String::NewSymbol("max"), Integer::New(value.Max));
    obj->Set(String::NewSymbol("min"), Integer::New(value.Min));
    obj->Set(String::NewSymbol("sum"), Integer::New(value.Sum));
    
    return scope.Close(obj);
  }

  
  static bool IsBandwidthStatisticsJsObject(Handle<Value> value)
  {
    if (!value->IsObject())
    {
      return false;
    }

    Handle<String> symbol;
    Handle<Object> obj = value.As<Object>();

    symbol = String::NewSymbol("outboundBitsPerSecond");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsNumber())
      {
          return false;
      }
    }
    
    symbol = String::NewSymbol("inboundBitsPerSecond");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsNumber())
      {
          return false;
      }
    }
    
    symbol = String::NewSymbol("outboundBitsPerSecondInstability");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsNumber())
      {
          return false;
      }
    }
    
    symbol = String::NewSymbol("inboundBitsPerSecondInstability");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsNumber())
      {
          return false;
      }
    }
    
    symbol = String::NewSymbol("outboundBandwidthPeaked");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsBoolean())
      {
          return false;
      }
    }
    
    symbol = String::NewSymbol("inboundBandwidthPeaked");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsBoolean())
      {
          return false;
      }
    }
    
    return true;
  }

  ::Windows::Networking::Sockets::BandwidthStatistics BandwidthStatisticsFromJsObject(Handle<Value> value)
  {
    HandleScope scope;
    ::Windows::Networking::Sockets::BandwidthStatistics returnValue;
    
    if (!value->IsObject())
    {
      ThrowException(Exception::TypeError(NodeRT::Utils::NewString(L"Unexpected type, expected an object")));
      return returnValue;
    }

    Handle<Object> obj = value.As<Object>();
    Handle<String> symbol;

    symbol = String::NewSymbol("outboundBitsPerSecond");
    if (obj->Has(symbol))
    {
      returnValue.OutboundBitsPerSecond = static_cast<unsigned __int64>(obj->Get(symbol)->IntegerValue());
    }
    
    symbol = String::NewSymbol("inboundBitsPerSecond");
    if (obj->Has(symbol))
    {
      returnValue.InboundBitsPerSecond = static_cast<unsigned __int64>(obj->Get(symbol)->IntegerValue());
    }
    
    symbol = String::NewSymbol("outboundBitsPerSecondInstability");
    if (obj->Has(symbol))
    {
      returnValue.OutboundBitsPerSecondInstability = static_cast<unsigned __int64>(obj->Get(symbol)->IntegerValue());
    }
    
    symbol = String::NewSymbol("inboundBitsPerSecondInstability");
    if (obj->Has(symbol))
    {
      returnValue.InboundBitsPerSecondInstability = static_cast<unsigned __int64>(obj->Get(symbol)->IntegerValue());
    }
    
    symbol = String::NewSymbol("outboundBandwidthPeaked");
    if (obj->Has(symbol))
    {
      returnValue.OutboundBandwidthPeaked = obj->Get(symbol)->BooleanValue();
    }
    
    symbol = String::NewSymbol("inboundBandwidthPeaked");
    if (obj->Has(symbol))
    {
      returnValue.InboundBandwidthPeaked = obj->Get(symbol)->BooleanValue();
    }
    
    return returnValue;
  }

  Handle<Value> BandwidthStatisticsToJsObject(::Windows::Networking::Sockets::BandwidthStatistics value)
  {
    HandleScope scope;

    Handle<Object> obj = Object::New();

    obj->Set(String::NewSymbol("outboundBitsPerSecond"), Number::New(static_cast<double>(value.OutboundBitsPerSecond)));
    obj->Set(String::NewSymbol("inboundBitsPerSecond"), Number::New(static_cast<double>(value.InboundBitsPerSecond)));
    obj->Set(String::NewSymbol("outboundBitsPerSecondInstability"), Number::New(static_cast<double>(value.OutboundBitsPerSecondInstability)));
    obj->Set(String::NewSymbol("inboundBitsPerSecondInstability"), Number::New(static_cast<double>(value.InboundBitsPerSecondInstability)));
    obj->Set(String::NewSymbol("outboundBandwidthPeaked"), Boolean::New(value.OutboundBandwidthPeaked));
    obj->Set(String::NewSymbol("inboundBandwidthPeaked"), Boolean::New(value.InboundBandwidthPeaked));
    
    return scope.Close(obj);
  }

  
  class ControlChannelTrigger : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ControlChannelTrigger"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("usingTransport"), FunctionTemplate::New(UsingTransport)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("waitForPushEnabled"), FunctionTemplate::New(WaitForPushEnabled)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("decreaseNetworkKeepAliveInterval"), FunctionTemplate::New(DecreaseNetworkKeepAliveInterval)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("flushTransport"), FunctionTemplate::New(FlushTransport)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("close"), FunctionTemplate::New(Close)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("serverKeepAliveIntervalInMinutes"), ServerKeepAliveIntervalInMinutesGetter, ServerKeepAliveIntervalInMinutesSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("controlChannelTriggerId"), ControlChannelTriggerIdGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("currentKeepAliveIntervalInMinutes"), CurrentKeepAliveIntervalInMinutesGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("keepAliveTrigger"), KeepAliveTriggerGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("pushNotificationTrigger"), PushNotificationTriggerGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("transportObject"), TransportObjectGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ControlChannelTrigger"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ControlChannelTrigger(::Windows::Networking::Sockets::ControlChannelTrigger^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::ControlChannelTrigger^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::ControlChannelTrigger^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && args[0]->IsString()
        && args[1]->IsUint32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          unsigned int arg1 = static_cast<unsigned int>(args[1]->IntegerValue());
          
          winRtInstance = ref new ::Windows::Networking::Sockets::ControlChannelTrigger(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && args[0]->IsString()
        && args[1]->IsUint32()
        && args[2]->IsInt32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          unsigned int arg1 = static_cast<unsigned int>(args[1]->IntegerValue());
          ::Windows::Networking::Sockets::ControlChannelTriggerResourceType arg2 = static_cast<::Windows::Networking::Sockets::ControlChannelTriggerResourceType>(args[2]->Int32Value());
          
          winRtInstance = ref new ::Windows::Networking::Sockets::ControlChannelTrigger(arg0,arg1,arg2);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ControlChannelTrigger *wrapperInstance = new ControlChannelTrigger(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> UsingTransport(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Platform::Object^>(args[0]))
      {
        try
        {
          ::Platform::Object^ arg0 = dynamic_cast<::Platform::Object^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          wrapper->_instance->UsingTransport(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> WaitForPushEnabled(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Networking::Sockets::ControlChannelTriggerStatus result;
          result = wrapper->_instance->WaitForPushEnabled();
          return scope.Close(Integer::New(static_cast<int>(result)));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> DecreaseNetworkKeepAliveInterval(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          wrapper->_instance->DecreaseNetworkKeepAliveInterval();
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> FlushTransport(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          wrapper->_instance->FlushTransport();
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Close(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          delete wrapper->_instance;
          wrapper->_instance = nullptr;
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }




    static Handle<Value> ServerKeepAliveIntervalInMinutesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->ServerKeepAliveIntervalInMinutes;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ServerKeepAliveIntervalInMinutesSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(info.This()))
      {
        return;
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->ServerKeepAliveIntervalInMinutes = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ControlChannelTriggerIdGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->ControlChannelTriggerId;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> CurrentKeepAliveIntervalInMinutesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->CurrentKeepAliveIntervalInMinutes;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> KeepAliveTriggerGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(info.This());

      try 
      {
        ::Windows::ApplicationModel::Background::IBackgroundTrigger^ result = wrapper->_instance->KeepAliveTrigger;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.ApplicationModel.Background", "IBackgroundTrigger", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> PushNotificationTriggerGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(info.This());

      try 
      {
        ::Windows::ApplicationModel::Background::IBackgroundTrigger^ result = wrapper->_instance->PushNotificationTrigger;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.ApplicationModel.Background", "IBackgroundTrigger", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> TransportObjectGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::ControlChannelTrigger^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ControlChannelTrigger *wrapper = ControlChannelTrigger::Unwrap<ControlChannelTrigger>(info.This());

      try 
      {
        ::Platform::Object^ result = wrapper->_instance->TransportObject;
        return scope.Close(CreateOpaqueWrapper(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::ControlChannelTrigger^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapControlChannelTrigger(::Windows::Networking::Sockets::ControlChannelTrigger^ wintRtInstance);
    friend ::Windows::Networking::Sockets::ControlChannelTrigger^ UnwrapControlChannelTrigger(Handle<Value> value);
    friend bool IsControlChannelTriggerWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ControlChannelTrigger::s_constructorTemplate;

  v8::Handle<v8::Value> WrapControlChannelTrigger(::Windows::Networking::Sockets::ControlChannelTrigger^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ControlChannelTrigger::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::ControlChannelTrigger^ UnwrapControlChannelTrigger(Handle<Value> value)
  {
     return ControlChannelTrigger::Unwrap<ControlChannelTrigger>(value.As<Object>())->_instance;
  }

  void InitControlChannelTrigger(Handle<Object> exports)
  {
    ControlChannelTrigger::Init(exports);
  }

  class IControlChannelTriggerEventDetails : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IControlChannelTriggerEventDetails"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("controlChannelTrigger"), ControlChannelTriggerGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IControlChannelTriggerEventDetails"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IControlChannelTriggerEventDetails(::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IControlChannelTriggerEventDetails *wrapperInstance = new IControlChannelTriggerEventDetails(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> ControlChannelTriggerGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IControlChannelTriggerEventDetails *wrapper = IControlChannelTriggerEventDetails::Unwrap<IControlChannelTriggerEventDetails>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::ControlChannelTrigger^ result = wrapper->_instance->ControlChannelTrigger;
        return scope.Close(WrapControlChannelTrigger(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIControlChannelTriggerEventDetails(::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^ wintRtInstance);
    friend ::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^ UnwrapIControlChannelTriggerEventDetails(Handle<Value> value);
    friend bool IsIControlChannelTriggerEventDetailsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IControlChannelTriggerEventDetails::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIControlChannelTriggerEventDetails(::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IControlChannelTriggerEventDetails::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::IControlChannelTriggerEventDetails^ UnwrapIControlChannelTriggerEventDetails(Handle<Value> value)
  {
     return IControlChannelTriggerEventDetails::Unwrap<IControlChannelTriggerEventDetails>(value.As<Object>())->_instance;
  }

  void InitIControlChannelTriggerEventDetails(Handle<Object> exports)
  {
    IControlChannelTriggerEventDetails::Init(exports);
  }

  class IControlChannelTriggerResetEventDetails : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IControlChannelTriggerResetEventDetails"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("hardwareSlotReset"), HardwareSlotResetGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("resetReason"), ResetReasonGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("softwareSlotReset"), SoftwareSlotResetGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IControlChannelTriggerResetEventDetails"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IControlChannelTriggerResetEventDetails(::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IControlChannelTriggerResetEventDetails *wrapperInstance = new IControlChannelTriggerResetEventDetails(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> HardwareSlotResetGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IControlChannelTriggerResetEventDetails *wrapper = IControlChannelTriggerResetEventDetails::Unwrap<IControlChannelTriggerResetEventDetails>(info.This());

      try 
      {
        bool result = wrapper->_instance->HardwareSlotReset;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ResetReasonGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IControlChannelTriggerResetEventDetails *wrapper = IControlChannelTriggerResetEventDetails::Unwrap<IControlChannelTriggerResetEventDetails>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::ControlChannelTriggerResetReason result = wrapper->_instance->ResetReason;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SoftwareSlotResetGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IControlChannelTriggerResetEventDetails *wrapper = IControlChannelTriggerResetEventDetails::Unwrap<IControlChannelTriggerResetEventDetails>(info.This());

      try 
      {
        bool result = wrapper->_instance->SoftwareSlotReset;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIControlChannelTriggerResetEventDetails(::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^ wintRtInstance);
    friend ::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^ UnwrapIControlChannelTriggerResetEventDetails(Handle<Value> value);
    friend bool IsIControlChannelTriggerResetEventDetailsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IControlChannelTriggerResetEventDetails::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIControlChannelTriggerResetEventDetails(::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IControlChannelTriggerResetEventDetails::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::IControlChannelTriggerResetEventDetails^ UnwrapIControlChannelTriggerResetEventDetails(Handle<Value> value)
  {
     return IControlChannelTriggerResetEventDetails::Unwrap<IControlChannelTriggerResetEventDetails>(value.As<Object>())->_instance;
  }

  void InitIControlChannelTriggerResetEventDetails(Handle<Object> exports)
  {
    IControlChannelTriggerResetEventDetails::Init(exports);
  }

  class DatagramSocketControl : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("DatagramSocketControl"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("inboundBufferSizeInBytes"), InboundBufferSizeInBytesGetter, InboundBufferSizeInBytesSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("dontFragment"), DontFragmentGetter, DontFragmentSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("qualityOfService"), QualityOfServiceGetter, QualityOfServiceSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outboundUnicastHopLimit"), OutboundUnicastHopLimitGetter, OutboundUnicastHopLimitSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("DatagramSocketControl"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    DatagramSocketControl(::Windows::Networking::Sockets::DatagramSocketControl^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::DatagramSocketControl^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketControl^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::DatagramSocketControl^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      DatagramSocketControl *wrapperInstance = new DatagramSocketControl(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> InboundBufferSizeInBytesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketControl *wrapper = DatagramSocketControl::Unwrap<DatagramSocketControl>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->InboundBufferSizeInBytes;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void InboundBufferSizeInBytesSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketControl^>(info.This()))
      {
        return;
      }

      DatagramSocketControl *wrapper = DatagramSocketControl::Unwrap<DatagramSocketControl>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->InboundBufferSizeInBytes = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> DontFragmentGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketControl *wrapper = DatagramSocketControl::Unwrap<DatagramSocketControl>(info.This());

      try 
      {
        bool result = wrapper->_instance->DontFragment;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void DontFragmentSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketControl^>(info.This()))
      {
        return;
      }

      DatagramSocketControl *wrapper = DatagramSocketControl::Unwrap<DatagramSocketControl>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->DontFragment = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> QualityOfServiceGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketControl *wrapper = DatagramSocketControl::Unwrap<DatagramSocketControl>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::SocketQualityOfService result = wrapper->_instance->QualityOfService;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void QualityOfServiceSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketControl^>(info.This()))
      {
        return;
      }

      DatagramSocketControl *wrapper = DatagramSocketControl::Unwrap<DatagramSocketControl>(info.This());

      try 
      {
        
        ::Windows::Networking::Sockets::SocketQualityOfService winRtValue = static_cast<::Windows::Networking::Sockets::SocketQualityOfService>(value->Int32Value());

        wrapper->_instance->QualityOfService = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> OutboundUnicastHopLimitGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketControl *wrapper = DatagramSocketControl::Unwrap<DatagramSocketControl>(info.This());

      try 
      {
        unsigned char result = wrapper->_instance->OutboundUnicastHopLimit;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void OutboundUnicastHopLimitSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketControl^>(info.This()))
      {
        return;
      }

      DatagramSocketControl *wrapper = DatagramSocketControl::Unwrap<DatagramSocketControl>(info.This());

      try 
      {
        
        unsigned char winRtValue = static_cast<unsigned char>(value->Int32Value());

        wrapper->_instance->OutboundUnicastHopLimit = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Networking::Sockets::DatagramSocketControl^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapDatagramSocketControl(::Windows::Networking::Sockets::DatagramSocketControl^ wintRtInstance);
    friend ::Windows::Networking::Sockets::DatagramSocketControl^ UnwrapDatagramSocketControl(Handle<Value> value);
    friend bool IsDatagramSocketControlWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> DatagramSocketControl::s_constructorTemplate;

  v8::Handle<v8::Value> WrapDatagramSocketControl(::Windows::Networking::Sockets::DatagramSocketControl^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(DatagramSocketControl::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::DatagramSocketControl^ UnwrapDatagramSocketControl(Handle<Value> value)
  {
     return DatagramSocketControl::Unwrap<DatagramSocketControl>(value.As<Object>())->_instance;
  }

  void InitDatagramSocketControl(Handle<Object> exports)
  {
    DatagramSocketControl::Init(exports);
  }

  class DatagramSocketInformation : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("DatagramSocketInformation"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("localAddress"), LocalAddressGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("localPort"), LocalPortGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("remoteAddress"), RemoteAddressGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("remotePort"), RemotePortGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("DatagramSocketInformation"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    DatagramSocketInformation(::Windows::Networking::Sockets::DatagramSocketInformation^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::DatagramSocketInformation^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketInformation^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::DatagramSocketInformation^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      DatagramSocketInformation *wrapperInstance = new DatagramSocketInformation(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> LocalAddressGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketInformation *wrapper = DatagramSocketInformation::Unwrap<DatagramSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::HostName^ result = wrapper->_instance->LocalAddress;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "HostName", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> LocalPortGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketInformation *wrapper = DatagramSocketInformation::Unwrap<DatagramSocketInformation>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->LocalPort;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> RemoteAddressGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketInformation *wrapper = DatagramSocketInformation::Unwrap<DatagramSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::HostName^ result = wrapper->_instance->RemoteAddress;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "HostName", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> RemotePortGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketInformation *wrapper = DatagramSocketInformation::Unwrap<DatagramSocketInformation>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->RemotePort;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::DatagramSocketInformation^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapDatagramSocketInformation(::Windows::Networking::Sockets::DatagramSocketInformation^ wintRtInstance);
    friend ::Windows::Networking::Sockets::DatagramSocketInformation^ UnwrapDatagramSocketInformation(Handle<Value> value);
    friend bool IsDatagramSocketInformationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> DatagramSocketInformation::s_constructorTemplate;

  v8::Handle<v8::Value> WrapDatagramSocketInformation(::Windows::Networking::Sockets::DatagramSocketInformation^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(DatagramSocketInformation::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::DatagramSocketInformation^ UnwrapDatagramSocketInformation(Handle<Value> value)
  {
     return DatagramSocketInformation::Unwrap<DatagramSocketInformation>(value.As<Object>())->_instance;
  }

  void InitDatagramSocketInformation(Handle<Object> exports)
  {
    DatagramSocketInformation::Init(exports);
  }

  class DatagramSocket : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("DatagramSocket"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("joinMulticastGroup"), FunctionTemplate::New(JoinMulticastGroup)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("close"), FunctionTemplate::New(Close)->GetFunction());
      
            
      func = FunctionTemplate::New(ConnectAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("connectAsync"), func);
      func = FunctionTemplate::New(BindServiceNameAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("bindServiceNameAsync"), func);
      func = FunctionTemplate::New(BindEndpointAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("bindEndpointAsync"), func);
      func = FunctionTemplate::New(GetOutputStreamAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getOutputStreamAsync"), func);
      
            
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("control"), ControlGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("information"), InformationGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outputStream"), OutputStreamGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      func = FunctionTemplate::New(GetEndpointPairsAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      constructor->Set(String::NewSymbol("getEndpointPairsAsync"), func);

      exports->Set(String::NewSymbol("DatagramSocket"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    DatagramSocket(::Windows::Networking::Sockets::DatagramSocket^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::DatagramSocket^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::DatagramSocket^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Networking::Sockets::DatagramSocket();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      DatagramSocket *wrapperInstance = new DatagramSocket(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> ConnectAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(args.This());

      ::Windows::Foundation::IAsyncAction^ op;
    

      if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[0])
        && args[1]->IsString())
      {
        try
        {
          ::Windows::Networking::HostName^ arg0 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          op = wrapper->_instance->ConnectAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::EndpointPair^>(args[0]))
      {
        try
        {
          ::Windows::Networking::EndpointPair^ arg0 = dynamic_cast<::Windows::Networking::EndpointPair^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          op = wrapper->_instance->ConnectAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<void> t) 
      {	
        try
        {
          t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            Handle<Value> args[] = {Undefined()};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
    static Handle<Value> BindServiceNameAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(args.This());

      ::Windows::Foundation::IAsyncAction^ op;
    

      if (args.Length() == 2
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          op = wrapper->_instance->BindServiceNameAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && args[0]->IsString()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Connectivity::NetworkAdapter^>(args[1]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Windows::Networking::Connectivity::NetworkAdapter^ arg1 = dynamic_cast<::Windows::Networking::Connectivity::NetworkAdapter^>(NodeRT::Utils::GetObjectInstance(args[1]));
          
          op = wrapper->_instance->BindServiceNameAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<void> t) 
      {	
        try
        {
          t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            Handle<Value> args[] = {Undefined()};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
    static Handle<Value> BindEndpointAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(args.This());

      ::Windows::Foundation::IAsyncAction^ op;
    

      if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[0])
        && args[1]->IsString())
      {
        try
        {
          ::Windows::Networking::HostName^ arg0 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          op = wrapper->_instance->BindEndpointAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<void> t) 
      {	
        try
        {
          t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            Handle<Value> args[] = {Undefined()};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
    static Handle<Value> GetOutputStreamAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(args.This());

      ::Windows::Foundation::IAsyncOperation<::Windows::Storage::Streams::IOutputStream^>^ op;
    

      if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[0])
        && args[1]->IsString())
      {
        try
        {
          ::Windows::Networking::HostName^ arg0 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          op = wrapper->_instance->GetOutputStreamAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::EndpointPair^>(args[0]))
      {
        try
        {
          ::Windows::Networking::EndpointPair^ arg0 = dynamic_cast<::Windows::Networking::EndpointPair^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          op = wrapper->_instance->GetOutputStreamAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Storage::Streams::IOutputStream^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IOutputStream", result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> JoinMulticastGroup(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[0]))
      {
        try
        {
          ::Windows::Networking::HostName^ arg0 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          wrapper->_instance->JoinMulticastGroup(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Close(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          delete wrapper->_instance;
          wrapper->_instance = nullptr;
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }


    static Handle<Value> GetEndpointPairsAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      ::Windows::Foundation::IAsyncOperation<::Windows::Foundation::Collections::IVectorView<::Windows::Networking::EndpointPair^>^>^ op;
      

      if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[0])
        && args[1]->IsString())
      {
        try
        {
          ::Windows::Networking::HostName^ arg0 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          op = ::Windows::Networking::Sockets::DatagramSocket::GetEndpointPairsAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 4
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[0])
        && args[1]->IsString()
        && args[2]->IsInt32())
      {
        try
        {
          ::Windows::Networking::HostName^ arg0 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          ::Windows::Networking::HostNameSortOptions arg2 = static_cast<::Windows::Networking::HostNameSortOptions>(args[2]->Int32Value());
          
          op = ::Windows::Networking::Sockets::DatagramSocket::GetEndpointPairsAsync(arg0,arg1,arg2);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Foundation::Collections::IVectorView<::Windows::Networking::EndpointPair^>^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {

            
            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = NodeRT::Collections::VectorViewWrapper<::Windows::Networking::EndpointPair^>::CreateVectorViewWrapper(result, 
            [](::Windows::Networking::EndpointPair^ val) -> Handle<Value> {
              return NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "EndpointPair", val);
            },
            [](Handle<Value> value) -> bool {
              return NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::EndpointPair^>(value);
            },
            [](Handle<Value> value) -> ::Windows::Networking::EndpointPair^ {
              return dynamic_cast<::Windows::Networking::EndpointPair^>(NodeRT::Utils::GetObjectInstance(value));
            }
          );
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
          
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }


    static Handle<Value> ControlGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::DatagramSocketControl^ result = wrapper->_instance->Control;
        return scope.Close(WrapDatagramSocketControl(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> InformationGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::DatagramSocketInformation^ result = wrapper->_instance->Information;
        return scope.Close(WrapDatagramSocketInformation(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> OutputStreamGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(info.This());

      try 
      {
        ::Windows::Storage::Streams::IOutputStream^ result = wrapper->_instance->OutputStream;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IOutputStream", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"messageReceived", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->MessageReceived::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Networking::Sockets::DatagramSocket^, ::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^>(
            [callbackObjPtr](::Windows::Networking::Sockets::DatagramSocket^ arg0, ::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapDatagramSocket(arg0);
                Handle<Value> wrappedArg1 = WrapDatagramSocketMessageReceivedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"messageReceived", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"messageReceived", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocket^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          DatagramSocket *wrapper = DatagramSocket::Unwrap<DatagramSocket>(args.This());
          wrapper->_instance->MessageReceived::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Networking::Sockets::DatagramSocket^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapDatagramSocket(::Windows::Networking::Sockets::DatagramSocket^ wintRtInstance);
    friend ::Windows::Networking::Sockets::DatagramSocket^ UnwrapDatagramSocket(Handle<Value> value);
    friend bool IsDatagramSocketWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> DatagramSocket::s_constructorTemplate;

  v8::Handle<v8::Value> WrapDatagramSocket(::Windows::Networking::Sockets::DatagramSocket^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(DatagramSocket::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::DatagramSocket^ UnwrapDatagramSocket(Handle<Value> value)
  {
     return DatagramSocket::Unwrap<DatagramSocket>(value.As<Object>())->_instance;
  }

  void InitDatagramSocket(Handle<Object> exports)
  {
    DatagramSocket::Init(exports);
  }

  class DatagramSocketMessageReceivedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("DatagramSocketMessageReceivedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getDataReader"), FunctionTemplate::New(GetDataReader)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getDataStream"), FunctionTemplate::New(GetDataStream)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("localAddress"), LocalAddressGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("remoteAddress"), RemoteAddressGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("remotePort"), RemotePortGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("DatagramSocketMessageReceivedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    DatagramSocketMessageReceivedEventArgs(::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      DatagramSocketMessageReceivedEventArgs *wrapperInstance = new DatagramSocketMessageReceivedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> GetDataReader(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketMessageReceivedEventArgs *wrapper = DatagramSocketMessageReceivedEventArgs::Unwrap<DatagramSocketMessageReceivedEventArgs>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Storage::Streams::DataReader^ result;
          result = wrapper->_instance->GetDataReader();
          return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "DataReader", result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetDataStream(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketMessageReceivedEventArgs *wrapper = DatagramSocketMessageReceivedEventArgs::Unwrap<DatagramSocketMessageReceivedEventArgs>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Storage::Streams::IInputStream^ result;
          result = wrapper->_instance->GetDataStream();
          return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IInputStream", result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> LocalAddressGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketMessageReceivedEventArgs *wrapper = DatagramSocketMessageReceivedEventArgs::Unwrap<DatagramSocketMessageReceivedEventArgs>(info.This());

      try 
      {
        ::Windows::Networking::HostName^ result = wrapper->_instance->LocalAddress;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "HostName", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> RemoteAddressGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketMessageReceivedEventArgs *wrapper = DatagramSocketMessageReceivedEventArgs::Unwrap<DatagramSocketMessageReceivedEventArgs>(info.This());

      try 
      {
        ::Windows::Networking::HostName^ result = wrapper->_instance->RemoteAddress;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "HostName", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> RemotePortGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      DatagramSocketMessageReceivedEventArgs *wrapper = DatagramSocketMessageReceivedEventArgs::Unwrap<DatagramSocketMessageReceivedEventArgs>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->RemotePort;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapDatagramSocketMessageReceivedEventArgs(::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^ wintRtInstance);
    friend ::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^ UnwrapDatagramSocketMessageReceivedEventArgs(Handle<Value> value);
    friend bool IsDatagramSocketMessageReceivedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> DatagramSocketMessageReceivedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapDatagramSocketMessageReceivedEventArgs(::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(DatagramSocketMessageReceivedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::DatagramSocketMessageReceivedEventArgs^ UnwrapDatagramSocketMessageReceivedEventArgs(Handle<Value> value)
  {
     return DatagramSocketMessageReceivedEventArgs::Unwrap<DatagramSocketMessageReceivedEventArgs>(value.As<Object>())->_instance;
  }

  void InitDatagramSocketMessageReceivedEventArgs(Handle<Object> exports)
  {
    DatagramSocketMessageReceivedEventArgs::Init(exports);
  }

  class StreamSocketControl : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("StreamSocketControl"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("qualityOfService"), QualityOfServiceGetter, QualityOfServiceSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outboundUnicastHopLimit"), OutboundUnicastHopLimitGetter, OutboundUnicastHopLimitSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outboundBufferSizeInBytes"), OutboundBufferSizeInBytesGetter, OutboundBufferSizeInBytesSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("noDelay"), NoDelayGetter, NoDelaySetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("keepAlive"), KeepAliveGetter, KeepAliveSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("ignorableServerCertificateErrors"), IgnorableServerCertificateErrorsGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("StreamSocketControl"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    StreamSocketControl(::Windows::Networking::Sockets::StreamSocketControl^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::StreamSocketControl^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::StreamSocketControl^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      StreamSocketControl *wrapperInstance = new StreamSocketControl(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> QualityOfServiceGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::SocketQualityOfService result = wrapper->_instance->QualityOfService;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void QualityOfServiceSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return;
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        
        ::Windows::Networking::Sockets::SocketQualityOfService winRtValue = static_cast<::Windows::Networking::Sockets::SocketQualityOfService>(value->Int32Value());

        wrapper->_instance->QualityOfService = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> OutboundUnicastHopLimitGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        unsigned char result = wrapper->_instance->OutboundUnicastHopLimit;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void OutboundUnicastHopLimitSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return;
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        
        unsigned char winRtValue = static_cast<unsigned char>(value->Int32Value());

        wrapper->_instance->OutboundUnicastHopLimit = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> OutboundBufferSizeInBytesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->OutboundBufferSizeInBytes;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void OutboundBufferSizeInBytesSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return;
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->OutboundBufferSizeInBytes = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> NoDelayGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        bool result = wrapper->_instance->NoDelay;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void NoDelaySetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return;
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->NoDelay = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> KeepAliveGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        bool result = wrapper->_instance->KeepAlive;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void KeepAliveSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return;
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->KeepAlive = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IgnorableServerCertificateErrorsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketControl *wrapper = StreamSocketControl::Unwrap<StreamSocketControl>(info.This());

      try 
      {
        ::Windows::Foundation::Collections::IVector<::Windows::Security::Cryptography::Certificates::ChainValidationResult>^ result = wrapper->_instance->IgnorableServerCertificateErrors;
        return scope.Close(NodeRT::Collections::VectorWrapper<::Windows::Security::Cryptography::Certificates::ChainValidationResult>::CreateVectorWrapper(result, 
            [](::Windows::Security::Cryptography::Certificates::ChainValidationResult val) -> Handle<Value> {
              return Integer::New(static_cast<int>(val));
            },
            [](Handle<Value> value) -> bool {
              return value->IsInt32();
            },
            [](Handle<Value> value) -> ::Windows::Security::Cryptography::Certificates::ChainValidationResult {
              return static_cast<::Windows::Security::Cryptography::Certificates::ChainValidationResult>(value->Int32Value());
            }
          ));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::StreamSocketControl^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapStreamSocketControl(::Windows::Networking::Sockets::StreamSocketControl^ wintRtInstance);
    friend ::Windows::Networking::Sockets::StreamSocketControl^ UnwrapStreamSocketControl(Handle<Value> value);
    friend bool IsStreamSocketControlWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> StreamSocketControl::s_constructorTemplate;

  v8::Handle<v8::Value> WrapStreamSocketControl(::Windows::Networking::Sockets::StreamSocketControl^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(StreamSocketControl::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::StreamSocketControl^ UnwrapStreamSocketControl(Handle<Value> value)
  {
     return StreamSocketControl::Unwrap<StreamSocketControl>(value.As<Object>())->_instance;
  }

  void InitStreamSocketControl(Handle<Object> exports)
  {
    StreamSocketControl::Init(exports);
  }

  class StreamSocketInformation : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("StreamSocketInformation"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("bandwidthStatistics"), BandwidthStatisticsGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("localAddress"), LocalAddressGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("localPort"), LocalPortGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("protectionLevel"), ProtectionLevelGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("remoteAddress"), RemoteAddressGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("remoteHostName"), RemoteHostNameGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("remotePort"), RemotePortGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("remoteServiceName"), RemoteServiceNameGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("roundTripTimeStatistics"), RoundTripTimeStatisticsGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("sessionKey"), SessionKeyGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("serverCertificate"), ServerCertificateGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("serverCertificateErrorSeverity"), ServerCertificateErrorSeverityGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("serverCertificateErrors"), ServerCertificateErrorsGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("serverIntermediateCertificates"), ServerIntermediateCertificatesGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("StreamSocketInformation"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    StreamSocketInformation(::Windows::Networking::Sockets::StreamSocketInformation^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::StreamSocketInformation^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::StreamSocketInformation^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      StreamSocketInformation *wrapperInstance = new StreamSocketInformation(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> BandwidthStatisticsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::BandwidthStatistics result = wrapper->_instance->BandwidthStatistics;
        return scope.Close(BandwidthStatisticsToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> LocalAddressGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::HostName^ result = wrapper->_instance->LocalAddress;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "HostName", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> LocalPortGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->LocalPort;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ProtectionLevelGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::SocketProtectionLevel result = wrapper->_instance->ProtectionLevel;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> RemoteAddressGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::HostName^ result = wrapper->_instance->RemoteAddress;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "HostName", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> RemoteHostNameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::HostName^ result = wrapper->_instance->RemoteHostName;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "HostName", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> RemotePortGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->RemotePort;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> RemoteServiceNameGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->RemoteServiceName;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> RoundTripTimeStatisticsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::RoundTripTimeStatistics result = wrapper->_instance->RoundTripTimeStatistics;
        return scope.Close(RoundTripTimeStatisticsToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SessionKeyGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Storage::Streams::IBuffer^ result = wrapper->_instance->SessionKey;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IBuffer", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ServerCertificateGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Security::Cryptography::Certificates::Certificate^ result = wrapper->_instance->ServerCertificate;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Security.Cryptography.Certificates", "Certificate", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ServerCertificateErrorSeverityGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::SocketSslErrorSeverity result = wrapper->_instance->ServerCertificateErrorSeverity;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ServerCertificateErrorsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Foundation::Collections::IVectorView<::Windows::Security::Cryptography::Certificates::ChainValidationResult>^ result = wrapper->_instance->ServerCertificateErrors;
        return scope.Close(NodeRT::Collections::VectorViewWrapper<::Windows::Security::Cryptography::Certificates::ChainValidationResult>::CreateVectorViewWrapper(result, 
            [](::Windows::Security::Cryptography::Certificates::ChainValidationResult val) -> Handle<Value> {
              return Integer::New(static_cast<int>(val));
            },
            [](Handle<Value> value) -> bool {
              return value->IsInt32();
            },
            [](Handle<Value> value) -> ::Windows::Security::Cryptography::Certificates::ChainValidationResult {
              return static_cast<::Windows::Security::Cryptography::Certificates::ChainValidationResult>(value->Int32Value());
            }
          ));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ServerIntermediateCertificatesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketInformation *wrapper = StreamSocketInformation::Unwrap<StreamSocketInformation>(info.This());

      try 
      {
        ::Windows::Foundation::Collections::IVectorView<::Windows::Security::Cryptography::Certificates::Certificate^>^ result = wrapper->_instance->ServerIntermediateCertificates;
        return scope.Close(NodeRT::Collections::VectorViewWrapper<::Windows::Security::Cryptography::Certificates::Certificate^>::CreateVectorViewWrapper(result, 
            [](::Windows::Security::Cryptography::Certificates::Certificate^ val) -> Handle<Value> {
              return NodeRT::Utils::CreateExternalWinRTObject("Windows.Security.Cryptography.Certificates", "Certificate", val);
            },
            [](Handle<Value> value) -> bool {
              return NodeRT::Utils::IsWinRtWrapperOf<::Windows::Security::Cryptography::Certificates::Certificate^>(value);
            },
            [](Handle<Value> value) -> ::Windows::Security::Cryptography::Certificates::Certificate^ {
              return dynamic_cast<::Windows::Security::Cryptography::Certificates::Certificate^>(NodeRT::Utils::GetObjectInstance(value));
            }
          ));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::StreamSocketInformation^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapStreamSocketInformation(::Windows::Networking::Sockets::StreamSocketInformation^ wintRtInstance);
    friend ::Windows::Networking::Sockets::StreamSocketInformation^ UnwrapStreamSocketInformation(Handle<Value> value);
    friend bool IsStreamSocketInformationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> StreamSocketInformation::s_constructorTemplate;

  v8::Handle<v8::Value> WrapStreamSocketInformation(::Windows::Networking::Sockets::StreamSocketInformation^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(StreamSocketInformation::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::StreamSocketInformation^ UnwrapStreamSocketInformation(Handle<Value> value)
  {
     return StreamSocketInformation::Unwrap<StreamSocketInformation>(value.As<Object>())->_instance;
  }

  void InitStreamSocketInformation(Handle<Object> exports)
  {
    StreamSocketInformation::Init(exports);
  }

  class StreamSocket : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("StreamSocket"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("close"), FunctionTemplate::New(Close)->GetFunction());
      
            
      func = FunctionTemplate::New(ConnectAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("connectAsync"), func);
      func = FunctionTemplate::New(UpgradeToSslAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("upgradeToSslAsync"), func);
      
                  
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("control"), ControlGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("information"), InformationGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("inputStream"), InputStreamGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outputStream"), OutputStreamGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("StreamSocket"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    StreamSocket(::Windows::Networking::Sockets::StreamSocket^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::StreamSocket^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocket^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::StreamSocket^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Networking::Sockets::StreamSocket();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      StreamSocket *wrapperInstance = new StreamSocket(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> ConnectAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      StreamSocket *wrapper = StreamSocket::Unwrap<StreamSocket>(args.This());

      ::Windows::Foundation::IAsyncAction^ op;
    

      if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::EndpointPair^>(args[0]))
      {
        try
        {
          ::Windows::Networking::EndpointPair^ arg0 = dynamic_cast<::Windows::Networking::EndpointPair^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          op = wrapper->_instance->ConnectAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[0])
        && args[1]->IsString())
      {
        try
        {
          ::Windows::Networking::HostName^ arg0 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          op = wrapper->_instance->ConnectAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::EndpointPair^>(args[0])
        && args[1]->IsInt32())
      {
        try
        {
          ::Windows::Networking::EndpointPair^ arg0 = dynamic_cast<::Windows::Networking::EndpointPair^>(NodeRT::Utils::GetObjectInstance(args[0]));
          ::Windows::Networking::Sockets::SocketProtectionLevel arg1 = static_cast<::Windows::Networking::Sockets::SocketProtectionLevel>(args[1]->Int32Value());
          
          op = wrapper->_instance->ConnectAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 4
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[0])
        && args[1]->IsString()
        && args[2]->IsInt32())
      {
        try
        {
          ::Windows::Networking::HostName^ arg0 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          ::Windows::Networking::Sockets::SocketProtectionLevel arg2 = static_cast<::Windows::Networking::Sockets::SocketProtectionLevel>(args[2]->Int32Value());
          
          op = wrapper->_instance->ConnectAsync(arg0,arg1,arg2);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 5
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[0])
        && args[1]->IsString()
        && args[2]->IsInt32()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Connectivity::NetworkAdapter^>(args[3]))
      {
        try
        {
          ::Windows::Networking::HostName^ arg0 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          ::Windows::Networking::Sockets::SocketProtectionLevel arg2 = static_cast<::Windows::Networking::Sockets::SocketProtectionLevel>(args[2]->Int32Value());
          ::Windows::Networking::Connectivity::NetworkAdapter^ arg3 = dynamic_cast<::Windows::Networking::Connectivity::NetworkAdapter^>(NodeRT::Utils::GetObjectInstance(args[3]));
          
          op = wrapper->_instance->ConnectAsync(arg0,arg1,arg2,arg3);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<void> t) 
      {	
        try
        {
          t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            Handle<Value> args[] = {Undefined()};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
    static Handle<Value> UpgradeToSslAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      StreamSocket *wrapper = StreamSocket::Unwrap<StreamSocket>(args.This());

      ::Windows::Foundation::IAsyncAction^ op;
    

      if (args.Length() == 3
        && args[0]->IsInt32()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[1]))
      {
        try
        {
          ::Windows::Networking::Sockets::SocketProtectionLevel arg0 = static_cast<::Windows::Networking::Sockets::SocketProtectionLevel>(args[0]->Int32Value());
          ::Windows::Networking::HostName^ arg1 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[1]));
          
          op = wrapper->_instance->UpgradeToSslAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<void> t) 
      {	
        try
        {
          t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            Handle<Value> args[] = {Undefined()};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> Close(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocket *wrapper = StreamSocket::Unwrap<StreamSocket>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          delete wrapper->_instance;
          wrapper->_instance = nullptr;
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }




    static Handle<Value> ControlGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocket *wrapper = StreamSocket::Unwrap<StreamSocket>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::StreamSocketControl^ result = wrapper->_instance->Control;
        return scope.Close(WrapStreamSocketControl(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> InformationGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocket *wrapper = StreamSocket::Unwrap<StreamSocket>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::StreamSocketInformation^ result = wrapper->_instance->Information;
        return scope.Close(WrapStreamSocketInformation(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> InputStreamGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocket *wrapper = StreamSocket::Unwrap<StreamSocket>(info.This());

      try 
      {
        ::Windows::Storage::Streams::IInputStream^ result = wrapper->_instance->InputStream;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IInputStream", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> OutputStreamGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocket *wrapper = StreamSocket::Unwrap<StreamSocket>(info.This());

      try 
      {
        ::Windows::Storage::Streams::IOutputStream^ result = wrapper->_instance->OutputStream;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IOutputStream", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::StreamSocket^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapStreamSocket(::Windows::Networking::Sockets::StreamSocket^ wintRtInstance);
    friend ::Windows::Networking::Sockets::StreamSocket^ UnwrapStreamSocket(Handle<Value> value);
    friend bool IsStreamSocketWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> StreamSocket::s_constructorTemplate;

  v8::Handle<v8::Value> WrapStreamSocket(::Windows::Networking::Sockets::StreamSocket^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(StreamSocket::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::StreamSocket^ UnwrapStreamSocket(Handle<Value> value)
  {
     return StreamSocket::Unwrap<StreamSocket>(value.As<Object>())->_instance;
  }

  void InitStreamSocket(Handle<Object> exports)
  {
    StreamSocket::Init(exports);
  }

  class StreamSocketListenerControl : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("StreamSocketListenerControl"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("qualityOfService"), QualityOfServiceGetter, QualityOfServiceSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("StreamSocketListenerControl"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    StreamSocketListenerControl(::Windows::Networking::Sockets::StreamSocketListenerControl^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::StreamSocketListenerControl^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListenerControl^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::StreamSocketListenerControl^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      StreamSocketListenerControl *wrapperInstance = new StreamSocketListenerControl(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> QualityOfServiceGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListenerControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketListenerControl *wrapper = StreamSocketListenerControl::Unwrap<StreamSocketListenerControl>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::SocketQualityOfService result = wrapper->_instance->QualityOfService;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void QualityOfServiceSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListenerControl^>(info.This()))
      {
        return;
      }

      StreamSocketListenerControl *wrapper = StreamSocketListenerControl::Unwrap<StreamSocketListenerControl>(info.This());

      try 
      {
        
        ::Windows::Networking::Sockets::SocketQualityOfService winRtValue = static_cast<::Windows::Networking::Sockets::SocketQualityOfService>(value->Int32Value());

        wrapper->_instance->QualityOfService = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Networking::Sockets::StreamSocketListenerControl^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapStreamSocketListenerControl(::Windows::Networking::Sockets::StreamSocketListenerControl^ wintRtInstance);
    friend ::Windows::Networking::Sockets::StreamSocketListenerControl^ UnwrapStreamSocketListenerControl(Handle<Value> value);
    friend bool IsStreamSocketListenerControlWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> StreamSocketListenerControl::s_constructorTemplate;

  v8::Handle<v8::Value> WrapStreamSocketListenerControl(::Windows::Networking::Sockets::StreamSocketListenerControl^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(StreamSocketListenerControl::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::StreamSocketListenerControl^ UnwrapStreamSocketListenerControl(Handle<Value> value)
  {
     return StreamSocketListenerControl::Unwrap<StreamSocketListenerControl>(value.As<Object>())->_instance;
  }

  void InitStreamSocketListenerControl(Handle<Object> exports)
  {
    StreamSocketListenerControl::Init(exports);
  }

  class StreamSocketListenerInformation : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("StreamSocketListenerInformation"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("localPort"), LocalPortGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("StreamSocketListenerInformation"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    StreamSocketListenerInformation(::Windows::Networking::Sockets::StreamSocketListenerInformation^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::StreamSocketListenerInformation^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListenerInformation^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::StreamSocketListenerInformation^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      StreamSocketListenerInformation *wrapperInstance = new StreamSocketListenerInformation(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> LocalPortGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListenerInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketListenerInformation *wrapper = StreamSocketListenerInformation::Unwrap<StreamSocketListenerInformation>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->LocalPort;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::StreamSocketListenerInformation^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapStreamSocketListenerInformation(::Windows::Networking::Sockets::StreamSocketListenerInformation^ wintRtInstance);
    friend ::Windows::Networking::Sockets::StreamSocketListenerInformation^ UnwrapStreamSocketListenerInformation(Handle<Value> value);
    friend bool IsStreamSocketListenerInformationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> StreamSocketListenerInformation::s_constructorTemplate;

  v8::Handle<v8::Value> WrapStreamSocketListenerInformation(::Windows::Networking::Sockets::StreamSocketListenerInformation^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(StreamSocketListenerInformation::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::StreamSocketListenerInformation^ UnwrapStreamSocketListenerInformation(Handle<Value> value)
  {
     return StreamSocketListenerInformation::Unwrap<StreamSocketListenerInformation>(value.As<Object>())->_instance;
  }

  void InitStreamSocketListenerInformation(Handle<Object> exports)
  {
    StreamSocketListenerInformation::Init(exports);
  }

  class StreamSocketListener : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("StreamSocketListener"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("close"), FunctionTemplate::New(Close)->GetFunction());
      
            
      func = FunctionTemplate::New(BindServiceNameAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("bindServiceNameAsync"), func);
      func = FunctionTemplate::New(BindEndpointAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("bindEndpointAsync"), func);
      
            
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("control"), ControlGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("information"), InformationGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("StreamSocketListener"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    StreamSocketListener(::Windows::Networking::Sockets::StreamSocketListener^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::StreamSocketListener^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListener^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::StreamSocketListener^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Networking::Sockets::StreamSocketListener();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      StreamSocketListener *wrapperInstance = new StreamSocketListener(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> BindServiceNameAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListener^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      StreamSocketListener *wrapper = StreamSocketListener::Unwrap<StreamSocketListener>(args.This());

      ::Windows::Foundation::IAsyncAction^ op;
    

      if (args.Length() == 2
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          op = wrapper->_instance->BindServiceNameAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && args[0]->IsString()
        && args[1]->IsInt32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Windows::Networking::Sockets::SocketProtectionLevel arg1 = static_cast<::Windows::Networking::Sockets::SocketProtectionLevel>(args[1]->Int32Value());
          
          op = wrapper->_instance->BindServiceNameAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 4
        && args[0]->IsString()
        && args[1]->IsInt32()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Connectivity::NetworkAdapter^>(args[2]))
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          ::Windows::Networking::Sockets::SocketProtectionLevel arg1 = static_cast<::Windows::Networking::Sockets::SocketProtectionLevel>(args[1]->Int32Value());
          ::Windows::Networking::Connectivity::NetworkAdapter^ arg2 = dynamic_cast<::Windows::Networking::Connectivity::NetworkAdapter^>(NodeRT::Utils::GetObjectInstance(args[2]));
          
          op = wrapper->_instance->BindServiceNameAsync(arg0,arg1,arg2);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<void> t) 
      {	
        try
        {
          t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            Handle<Value> args[] = {Undefined()};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
    static Handle<Value> BindEndpointAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListener^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      StreamSocketListener *wrapper = StreamSocketListener::Unwrap<StreamSocketListener>(args.This());

      ::Windows::Foundation::IAsyncAction^ op;
    

      if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::HostName^>(args[0])
        && args[1]->IsString())
      {
        try
        {
          ::Windows::Networking::HostName^ arg0 = dynamic_cast<::Windows::Networking::HostName^>(NodeRT::Utils::GetObjectInstance(args[0]));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          op = wrapper->_instance->BindEndpointAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<void> t) 
      {	
        try
        {
          t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            Handle<Value> args[] = {Undefined()};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> Close(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListener^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketListener *wrapper = StreamSocketListener::Unwrap<StreamSocketListener>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          delete wrapper->_instance;
          wrapper->_instance = nullptr;
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }




    static Handle<Value> ControlGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListener^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketListener *wrapper = StreamSocketListener::Unwrap<StreamSocketListener>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::StreamSocketListenerControl^ result = wrapper->_instance->Control;
        return scope.Close(WrapStreamSocketListenerControl(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> InformationGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListener^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketListener *wrapper = StreamSocketListener::Unwrap<StreamSocketListener>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::StreamSocketListenerInformation^ result = wrapper->_instance->Information;
        return scope.Close(WrapStreamSocketListenerInformation(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"connectionReceived", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListener^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        StreamSocketListener *wrapper = StreamSocketListener::Unwrap<StreamSocketListener>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->ConnectionReceived::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Networking::Sockets::StreamSocketListener^, ::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^>(
            [callbackObjPtr](::Windows::Networking::Sockets::StreamSocketListener^ arg0, ::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapStreamSocketListener(arg0);
                Handle<Value> wrappedArg1 = WrapStreamSocketListenerConnectionReceivedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"connectionReceived", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"connectionReceived", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListener^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          StreamSocketListener *wrapper = StreamSocketListener::Unwrap<StreamSocketListener>(args.This());
          wrapper->_instance->ConnectionReceived::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Networking::Sockets::StreamSocketListener^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapStreamSocketListener(::Windows::Networking::Sockets::StreamSocketListener^ wintRtInstance);
    friend ::Windows::Networking::Sockets::StreamSocketListener^ UnwrapStreamSocketListener(Handle<Value> value);
    friend bool IsStreamSocketListenerWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> StreamSocketListener::s_constructorTemplate;

  v8::Handle<v8::Value> WrapStreamSocketListener(::Windows::Networking::Sockets::StreamSocketListener^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(StreamSocketListener::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::StreamSocketListener^ UnwrapStreamSocketListener(Handle<Value> value)
  {
     return StreamSocketListener::Unwrap<StreamSocketListener>(value.As<Object>())->_instance;
  }

  void InitStreamSocketListener(Handle<Object> exports)
  {
    StreamSocketListener::Init(exports);
  }

  class StreamSocketListenerConnectionReceivedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("StreamSocketListenerConnectionReceivedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("socket"), SocketGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("StreamSocketListenerConnectionReceivedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    StreamSocketListenerConnectionReceivedEventArgs(::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      StreamSocketListenerConnectionReceivedEventArgs *wrapperInstance = new StreamSocketListenerConnectionReceivedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> SocketGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamSocketListenerConnectionReceivedEventArgs *wrapper = StreamSocketListenerConnectionReceivedEventArgs::Unwrap<StreamSocketListenerConnectionReceivedEventArgs>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::StreamSocket^ result = wrapper->_instance->Socket;
        return scope.Close(WrapStreamSocket(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapStreamSocketListenerConnectionReceivedEventArgs(::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^ wintRtInstance);
    friend ::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^ UnwrapStreamSocketListenerConnectionReceivedEventArgs(Handle<Value> value);
    friend bool IsStreamSocketListenerConnectionReceivedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> StreamSocketListenerConnectionReceivedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapStreamSocketListenerConnectionReceivedEventArgs(::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(StreamSocketListenerConnectionReceivedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::StreamSocketListenerConnectionReceivedEventArgs^ UnwrapStreamSocketListenerConnectionReceivedEventArgs(Handle<Value> value)
  {
     return StreamSocketListenerConnectionReceivedEventArgs::Unwrap<StreamSocketListenerConnectionReceivedEventArgs>(value.As<Object>())->_instance;
  }

  void InitStreamSocketListenerConnectionReceivedEventArgs(Handle<Object> exports)
  {
    StreamSocketListenerConnectionReceivedEventArgs::Init(exports);
  }

  class IWebSocketControl : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IWebSocketControl"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outboundBufferSizeInBytes"), OutboundBufferSizeInBytesGetter, OutboundBufferSizeInBytesSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("proxyCredential"), ProxyCredentialGetter, ProxyCredentialSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("serverCredential"), ServerCredentialGetter, ServerCredentialSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("supportedProtocols"), SupportedProtocolsGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IWebSocketControl"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IWebSocketControl(::Windows::Networking::Sockets::IWebSocketControl^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::IWebSocketControl^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketControl^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::IWebSocketControl^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IWebSocketControl *wrapperInstance = new IWebSocketControl(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> OutboundBufferSizeInBytesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IWebSocketControl *wrapper = IWebSocketControl::Unwrap<IWebSocketControl>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->OutboundBufferSizeInBytes;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void OutboundBufferSizeInBytesSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketControl^>(info.This()))
      {
        return;
      }

      IWebSocketControl *wrapper = IWebSocketControl::Unwrap<IWebSocketControl>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->OutboundBufferSizeInBytes = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ProxyCredentialGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IWebSocketControl *wrapper = IWebSocketControl::Unwrap<IWebSocketControl>(info.This());

      try 
      {
        ::Windows::Security::Credentials::PasswordCredential^ result = wrapper->_instance->ProxyCredential;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Security.Credentials", "PasswordCredential", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ProxyCredentialSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Security::Credentials::PasswordCredential^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketControl^>(info.This()))
      {
        return;
      }

      IWebSocketControl *wrapper = IWebSocketControl::Unwrap<IWebSocketControl>(info.This());

      try 
      {
        
        ::Windows::Security::Credentials::PasswordCredential^ winRtValue = dynamic_cast<::Windows::Security::Credentials::PasswordCredential^>(NodeRT::Utils::GetObjectInstance(value));

        wrapper->_instance->ProxyCredential = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ServerCredentialGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IWebSocketControl *wrapper = IWebSocketControl::Unwrap<IWebSocketControl>(info.This());

      try 
      {
        ::Windows::Security::Credentials::PasswordCredential^ result = wrapper->_instance->ServerCredential;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Security.Credentials", "PasswordCredential", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ServerCredentialSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Security::Credentials::PasswordCredential^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketControl^>(info.This()))
      {
        return;
      }

      IWebSocketControl *wrapper = IWebSocketControl::Unwrap<IWebSocketControl>(info.This());

      try 
      {
        
        ::Windows::Security::Credentials::PasswordCredential^ winRtValue = dynamic_cast<::Windows::Security::Credentials::PasswordCredential^>(NodeRT::Utils::GetObjectInstance(value));

        wrapper->_instance->ServerCredential = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> SupportedProtocolsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IWebSocketControl *wrapper = IWebSocketControl::Unwrap<IWebSocketControl>(info.This());

      try 
      {
        ::Windows::Foundation::Collections::IVector<::Platform::String^>^ result = wrapper->_instance->SupportedProtocols;
        return scope.Close(NodeRT::Collections::VectorWrapper<::Platform::String^>::CreateVectorWrapper(result, 
            [](::Platform::String^ val) -> Handle<Value> {
              return NodeRT::Utils::NewString(val->Data());
            },
            [](Handle<Value> value) -> bool {
              return value->IsString();
            },
            [](Handle<Value> value) -> ::Platform::String^ {
              return ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));
            }
          ));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::IWebSocketControl^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIWebSocketControl(::Windows::Networking::Sockets::IWebSocketControl^ wintRtInstance);
    friend ::Windows::Networking::Sockets::IWebSocketControl^ UnwrapIWebSocketControl(Handle<Value> value);
    friend bool IsIWebSocketControlWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IWebSocketControl::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIWebSocketControl(::Windows::Networking::Sockets::IWebSocketControl^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IWebSocketControl::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::IWebSocketControl^ UnwrapIWebSocketControl(Handle<Value> value)
  {
     return IWebSocketControl::Unwrap<IWebSocketControl>(value.As<Object>())->_instance;
  }

  void InitIWebSocketControl(Handle<Object> exports)
  {
    IWebSocketControl::Init(exports);
  }

  class IWebSocketInformation : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IWebSocketInformation"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("bandwidthStatistics"), BandwidthStatisticsGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("localAddress"), LocalAddressGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("protocol"), ProtocolGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IWebSocketInformation"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IWebSocketInformation(::Windows::Networking::Sockets::IWebSocketInformation^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::IWebSocketInformation^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketInformation^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::IWebSocketInformation^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IWebSocketInformation *wrapperInstance = new IWebSocketInformation(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> BandwidthStatisticsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IWebSocketInformation *wrapper = IWebSocketInformation::Unwrap<IWebSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::BandwidthStatistics result = wrapper->_instance->BandwidthStatistics;
        return scope.Close(BandwidthStatisticsToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> LocalAddressGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IWebSocketInformation *wrapper = IWebSocketInformation::Unwrap<IWebSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::HostName^ result = wrapper->_instance->LocalAddress;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "HostName", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ProtocolGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IWebSocketInformation *wrapper = IWebSocketInformation::Unwrap<IWebSocketInformation>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Protocol;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::IWebSocketInformation^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIWebSocketInformation(::Windows::Networking::Sockets::IWebSocketInformation^ wintRtInstance);
    friend ::Windows::Networking::Sockets::IWebSocketInformation^ UnwrapIWebSocketInformation(Handle<Value> value);
    friend bool IsIWebSocketInformationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IWebSocketInformation::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIWebSocketInformation(::Windows::Networking::Sockets::IWebSocketInformation^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IWebSocketInformation::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::IWebSocketInformation^ UnwrapIWebSocketInformation(Handle<Value> value)
  {
     return IWebSocketInformation::Unwrap<IWebSocketInformation>(value.As<Object>())->_instance;
  }

  void InitIWebSocketInformation(Handle<Object> exports)
  {
    IWebSocketInformation::Init(exports);
  }

  class IWebSocket : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IWebSocket"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("setRequestHeader"), FunctionTemplate::New(SetRequestHeader)->GetFunction());
      
            
      func = FunctionTemplate::New(ConnectAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("connectAsync"), func);
      
            
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outputStream"), OutputStreamGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IWebSocket"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IWebSocket(::Windows::Networking::Sockets::IWebSocket^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::IWebSocket^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocket^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::IWebSocket^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IWebSocket *wrapperInstance = new IWebSocket(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> ConnectAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      IWebSocket *wrapper = IWebSocket::Unwrap<IWebSocket>(args.This());

      ::Windows::Foundation::IAsyncAction^ op;
    

      if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Uri^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Uri^ arg0 = dynamic_cast<::Windows::Foundation::Uri^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          op = wrapper->_instance->ConnectAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<void> t) 
      {	
        try
        {
          t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            Handle<Value> args[] = {Undefined()};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> SetRequestHeader(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      IWebSocket *wrapper = IWebSocket::Unwrap<IWebSocket>(args.This());

      if (args.Length() == 2
        && args[0]->IsString()
        && args[1]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          wrapper->_instance->SetRequestHeader(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> OutputStreamGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IWebSocket *wrapper = IWebSocket::Unwrap<IWebSocket>(info.This());

      try 
      {
        ::Windows::Storage::Streams::IOutputStream^ result = wrapper->_instance->OutputStream;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IOutputStream", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"closed", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocket^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        IWebSocket *wrapper = IWebSocket::Unwrap<IWebSocket>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->Closed::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Networking::Sockets::IWebSocket^, ::Windows::Networking::Sockets::WebSocketClosedEventArgs^>(
            [callbackObjPtr](::Windows::Networking::Sockets::IWebSocket^ arg0, ::Windows::Networking::Sockets::WebSocketClosedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapIWebSocket(arg0);
                Handle<Value> wrappedArg1 = WrapWebSocketClosedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"closed", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"closed", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::IWebSocket^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          IWebSocket *wrapper = IWebSocket::Unwrap<IWebSocket>(args.This());
          wrapper->_instance->Closed::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Networking::Sockets::IWebSocket^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIWebSocket(::Windows::Networking::Sockets::IWebSocket^ wintRtInstance);
    friend ::Windows::Networking::Sockets::IWebSocket^ UnwrapIWebSocket(Handle<Value> value);
    friend bool IsIWebSocketWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IWebSocket::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIWebSocket(::Windows::Networking::Sockets::IWebSocket^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IWebSocket::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::IWebSocket^ UnwrapIWebSocket(Handle<Value> value)
  {
     return IWebSocket::Unwrap<IWebSocket>(value.As<Object>())->_instance;
  }

  void InitIWebSocket(Handle<Object> exports)
  {
    IWebSocket::Init(exports);
  }

  class WebSocketClosedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("WebSocketClosedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("code"), CodeGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("reason"), ReasonGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("WebSocketClosedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    WebSocketClosedEventArgs(::Windows::Networking::Sockets::WebSocketClosedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::WebSocketClosedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::WebSocketClosedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::WebSocketClosedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      WebSocketClosedEventArgs *wrapperInstance = new WebSocketClosedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> CodeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::WebSocketClosedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      WebSocketClosedEventArgs *wrapper = WebSocketClosedEventArgs::Unwrap<WebSocketClosedEventArgs>(info.This());

      try 
      {
        unsigned short result = wrapper->_instance->Code;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ReasonGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::WebSocketClosedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      WebSocketClosedEventArgs *wrapper = WebSocketClosedEventArgs::Unwrap<WebSocketClosedEventArgs>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Reason;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::WebSocketClosedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapWebSocketClosedEventArgs(::Windows::Networking::Sockets::WebSocketClosedEventArgs^ wintRtInstance);
    friend ::Windows::Networking::Sockets::WebSocketClosedEventArgs^ UnwrapWebSocketClosedEventArgs(Handle<Value> value);
    friend bool IsWebSocketClosedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> WebSocketClosedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapWebSocketClosedEventArgs(::Windows::Networking::Sockets::WebSocketClosedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(WebSocketClosedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::WebSocketClosedEventArgs^ UnwrapWebSocketClosedEventArgs(Handle<Value> value)
  {
     return WebSocketClosedEventArgs::Unwrap<WebSocketClosedEventArgs>(value.As<Object>())->_instance;
  }

  void InitWebSocketClosedEventArgs(Handle<Object> exports)
  {
    WebSocketClosedEventArgs::Init(exports);
  }

  class MessageWebSocketControl : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MessageWebSocketControl"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("messageType"), MessageTypeGetter, MessageTypeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxMessageSize"), MaxMessageSizeGetter, MaxMessageSizeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("serverCredential"), ServerCredentialGetter, ServerCredentialSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("proxyCredential"), ProxyCredentialGetter, ProxyCredentialSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outboundBufferSizeInBytes"), OutboundBufferSizeInBytesGetter, OutboundBufferSizeInBytesSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("supportedProtocols"), SupportedProtocolsGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("MessageWebSocketControl"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MessageWebSocketControl(::Windows::Networking::Sockets::MessageWebSocketControl^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::MessageWebSocketControl^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::MessageWebSocketControl^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MessageWebSocketControl *wrapperInstance = new MessageWebSocketControl(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> MessageTypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::SocketMessageType result = wrapper->_instance->MessageType;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void MessageTypeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return;
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        
        ::Windows::Networking::Sockets::SocketMessageType winRtValue = static_cast<::Windows::Networking::Sockets::SocketMessageType>(value->Int32Value());

        wrapper->_instance->MessageType = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> MaxMessageSizeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->MaxMessageSize;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void MaxMessageSizeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return;
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->MaxMessageSize = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ServerCredentialGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        ::Windows::Security::Credentials::PasswordCredential^ result = wrapper->_instance->ServerCredential;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Security.Credentials", "PasswordCredential", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ServerCredentialSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Security::Credentials::PasswordCredential^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return;
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        
        ::Windows::Security::Credentials::PasswordCredential^ winRtValue = dynamic_cast<::Windows::Security::Credentials::PasswordCredential^>(NodeRT::Utils::GetObjectInstance(value));

        wrapper->_instance->ServerCredential = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ProxyCredentialGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        ::Windows::Security::Credentials::PasswordCredential^ result = wrapper->_instance->ProxyCredential;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Security.Credentials", "PasswordCredential", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ProxyCredentialSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Security::Credentials::PasswordCredential^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return;
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        
        ::Windows::Security::Credentials::PasswordCredential^ winRtValue = dynamic_cast<::Windows::Security::Credentials::PasswordCredential^>(NodeRT::Utils::GetObjectInstance(value));

        wrapper->_instance->ProxyCredential = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> OutboundBufferSizeInBytesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->OutboundBufferSizeInBytes;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void OutboundBufferSizeInBytesSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return;
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->OutboundBufferSizeInBytes = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> SupportedProtocolsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketControl *wrapper = MessageWebSocketControl::Unwrap<MessageWebSocketControl>(info.This());

      try 
      {
        ::Windows::Foundation::Collections::IVector<::Platform::String^>^ result = wrapper->_instance->SupportedProtocols;
        return scope.Close(NodeRT::Collections::VectorWrapper<::Platform::String^>::CreateVectorWrapper(result, 
            [](::Platform::String^ val) -> Handle<Value> {
              return NodeRT::Utils::NewString(val->Data());
            },
            [](Handle<Value> value) -> bool {
              return value->IsString();
            },
            [](Handle<Value> value) -> ::Platform::String^ {
              return ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));
            }
          ));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::MessageWebSocketControl^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMessageWebSocketControl(::Windows::Networking::Sockets::MessageWebSocketControl^ wintRtInstance);
    friend ::Windows::Networking::Sockets::MessageWebSocketControl^ UnwrapMessageWebSocketControl(Handle<Value> value);
    friend bool IsMessageWebSocketControlWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MessageWebSocketControl::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMessageWebSocketControl(::Windows::Networking::Sockets::MessageWebSocketControl^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MessageWebSocketControl::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::MessageWebSocketControl^ UnwrapMessageWebSocketControl(Handle<Value> value)
  {
     return MessageWebSocketControl::Unwrap<MessageWebSocketControl>(value.As<Object>())->_instance;
  }

  void InitMessageWebSocketControl(Handle<Object> exports)
  {
    MessageWebSocketControl::Init(exports);
  }

  class MessageWebSocketInformation : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MessageWebSocketInformation"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("bandwidthStatistics"), BandwidthStatisticsGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("localAddress"), LocalAddressGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("protocol"), ProtocolGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("MessageWebSocketInformation"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MessageWebSocketInformation(::Windows::Networking::Sockets::MessageWebSocketInformation^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::MessageWebSocketInformation^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketInformation^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::MessageWebSocketInformation^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MessageWebSocketInformation *wrapperInstance = new MessageWebSocketInformation(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> BandwidthStatisticsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketInformation *wrapper = MessageWebSocketInformation::Unwrap<MessageWebSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::BandwidthStatistics result = wrapper->_instance->BandwidthStatistics;
        return scope.Close(BandwidthStatisticsToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> LocalAddressGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketInformation *wrapper = MessageWebSocketInformation::Unwrap<MessageWebSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::HostName^ result = wrapper->_instance->LocalAddress;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "HostName", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ProtocolGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketInformation *wrapper = MessageWebSocketInformation::Unwrap<MessageWebSocketInformation>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Protocol;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::MessageWebSocketInformation^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMessageWebSocketInformation(::Windows::Networking::Sockets::MessageWebSocketInformation^ wintRtInstance);
    friend ::Windows::Networking::Sockets::MessageWebSocketInformation^ UnwrapMessageWebSocketInformation(Handle<Value> value);
    friend bool IsMessageWebSocketInformationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MessageWebSocketInformation::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMessageWebSocketInformation(::Windows::Networking::Sockets::MessageWebSocketInformation^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MessageWebSocketInformation::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::MessageWebSocketInformation^ UnwrapMessageWebSocketInformation(Handle<Value> value)
  {
     return MessageWebSocketInformation::Unwrap<MessageWebSocketInformation>(value.As<Object>())->_instance;
  }

  void InitMessageWebSocketInformation(Handle<Object> exports)
  {
    MessageWebSocketInformation::Init(exports);
  }

  class MessageWebSocket : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MessageWebSocket"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("setRequestHeader"), FunctionTemplate::New(SetRequestHeader)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("close"), FunctionTemplate::New(Close)->GetFunction());
      
            
      func = FunctionTemplate::New(ConnectAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("connectAsync"), func);
      
            
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("control"), ControlGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("information"), InformationGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outputStream"), OutputStreamGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("MessageWebSocket"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MessageWebSocket(::Windows::Networking::Sockets::MessageWebSocket^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::MessageWebSocket^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::MessageWebSocket^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Networking::Sockets::MessageWebSocket();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MessageWebSocket *wrapperInstance = new MessageWebSocket(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> ConnectAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      MessageWebSocket *wrapper = MessageWebSocket::Unwrap<MessageWebSocket>(args.This());

      ::Windows::Foundation::IAsyncAction^ op;
    

      if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Uri^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Uri^ arg0 = dynamic_cast<::Windows::Foundation::Uri^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          op = wrapper->_instance->ConnectAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<void> t) 
      {	
        try
        {
          t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            Handle<Value> args[] = {Undefined()};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> SetRequestHeader(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocket *wrapper = MessageWebSocket::Unwrap<MessageWebSocket>(args.This());

      if (args.Length() == 2
        && args[0]->IsString()
        && args[1]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          wrapper->_instance->SetRequestHeader(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Close(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocket *wrapper = MessageWebSocket::Unwrap<MessageWebSocket>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          delete wrapper->_instance;
          wrapper->_instance = nullptr;
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }




    static Handle<Value> ControlGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocket *wrapper = MessageWebSocket::Unwrap<MessageWebSocket>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::MessageWebSocketControl^ result = wrapper->_instance->Control;
        return scope.Close(WrapMessageWebSocketControl(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> InformationGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocket *wrapper = MessageWebSocket::Unwrap<MessageWebSocket>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::MessageWebSocketInformation^ result = wrapper->_instance->Information;
        return scope.Close(WrapMessageWebSocketInformation(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> OutputStreamGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocket *wrapper = MessageWebSocket::Unwrap<MessageWebSocket>(info.This());

      try 
      {
        ::Windows::Storage::Streams::IOutputStream^ result = wrapper->_instance->OutputStream;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IOutputStream", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"messageReceived", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        MessageWebSocket *wrapper = MessageWebSocket::Unwrap<MessageWebSocket>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->MessageReceived::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Networking::Sockets::MessageWebSocket^, ::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^>(
            [callbackObjPtr](::Windows::Networking::Sockets::MessageWebSocket^ arg0, ::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapMessageWebSocket(arg0);
                Handle<Value> wrappedArg1 = WrapMessageWebSocketMessageReceivedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"closed", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        MessageWebSocket *wrapper = MessageWebSocket::Unwrap<MessageWebSocket>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->Closed::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Networking::Sockets::IWebSocket^, ::Windows::Networking::Sockets::WebSocketClosedEventArgs^>(
            [callbackObjPtr](::Windows::Networking::Sockets::IWebSocket^ arg0, ::Windows::Networking::Sockets::WebSocketClosedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapIWebSocket(arg0);
                Handle<Value> wrappedArg1 = WrapWebSocketClosedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"messageReceived", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"closed", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"messageReceived", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          MessageWebSocket *wrapper = MessageWebSocket::Unwrap<MessageWebSocket>(args.This());
          wrapper->_instance->MessageReceived::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"closed", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocket^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          MessageWebSocket *wrapper = MessageWebSocket::Unwrap<MessageWebSocket>(args.This());
          wrapper->_instance->Closed::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Networking::Sockets::MessageWebSocket^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMessageWebSocket(::Windows::Networking::Sockets::MessageWebSocket^ wintRtInstance);
    friend ::Windows::Networking::Sockets::MessageWebSocket^ UnwrapMessageWebSocket(Handle<Value> value);
    friend bool IsMessageWebSocketWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MessageWebSocket::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMessageWebSocket(::Windows::Networking::Sockets::MessageWebSocket^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MessageWebSocket::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::MessageWebSocket^ UnwrapMessageWebSocket(Handle<Value> value)
  {
     return MessageWebSocket::Unwrap<MessageWebSocket>(value.As<Object>())->_instance;
  }

  void InitMessageWebSocket(Handle<Object> exports)
  {
    MessageWebSocket::Init(exports);
  }

  class MessageWebSocketMessageReceivedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MessageWebSocketMessageReceivedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getDataReader"), FunctionTemplate::New(GetDataReader)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getDataStream"), FunctionTemplate::New(GetDataStream)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("messageType"), MessageTypeGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("MessageWebSocketMessageReceivedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MessageWebSocketMessageReceivedEventArgs(::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MessageWebSocketMessageReceivedEventArgs *wrapperInstance = new MessageWebSocketMessageReceivedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> GetDataReader(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketMessageReceivedEventArgs *wrapper = MessageWebSocketMessageReceivedEventArgs::Unwrap<MessageWebSocketMessageReceivedEventArgs>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Storage::Streams::DataReader^ result;
          result = wrapper->_instance->GetDataReader();
          return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "DataReader", result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetDataStream(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketMessageReceivedEventArgs *wrapper = MessageWebSocketMessageReceivedEventArgs::Unwrap<MessageWebSocketMessageReceivedEventArgs>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Storage::Streams::IInputStream^ result;
          result = wrapper->_instance->GetDataStream();
          return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IInputStream", result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> MessageTypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MessageWebSocketMessageReceivedEventArgs *wrapper = MessageWebSocketMessageReceivedEventArgs::Unwrap<MessageWebSocketMessageReceivedEventArgs>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::SocketMessageType result = wrapper->_instance->MessageType;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMessageWebSocketMessageReceivedEventArgs(::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^ wintRtInstance);
    friend ::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^ UnwrapMessageWebSocketMessageReceivedEventArgs(Handle<Value> value);
    friend bool IsMessageWebSocketMessageReceivedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MessageWebSocketMessageReceivedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMessageWebSocketMessageReceivedEventArgs(::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MessageWebSocketMessageReceivedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::MessageWebSocketMessageReceivedEventArgs^ UnwrapMessageWebSocketMessageReceivedEventArgs(Handle<Value> value)
  {
     return MessageWebSocketMessageReceivedEventArgs::Unwrap<MessageWebSocketMessageReceivedEventArgs>(value.As<Object>())->_instance;
  }

  void InitMessageWebSocketMessageReceivedEventArgs(Handle<Object> exports)
  {
    MessageWebSocketMessageReceivedEventArgs::Init(exports);
  }

  class StreamWebSocketControl : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("StreamWebSocketControl"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("noDelay"), NoDelayGetter, NoDelaySetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("serverCredential"), ServerCredentialGetter, ServerCredentialSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("proxyCredential"), ProxyCredentialGetter, ProxyCredentialSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outboundBufferSizeInBytes"), OutboundBufferSizeInBytesGetter, OutboundBufferSizeInBytesSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("supportedProtocols"), SupportedProtocolsGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("StreamWebSocketControl"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    StreamWebSocketControl(::Windows::Networking::Sockets::StreamWebSocketControl^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::StreamWebSocketControl^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketControl^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::StreamWebSocketControl^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      StreamWebSocketControl *wrapperInstance = new StreamWebSocketControl(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> NoDelayGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocketControl *wrapper = StreamWebSocketControl::Unwrap<StreamWebSocketControl>(info.This());

      try 
      {
        bool result = wrapper->_instance->NoDelay;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void NoDelaySetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketControl^>(info.This()))
      {
        return;
      }

      StreamWebSocketControl *wrapper = StreamWebSocketControl::Unwrap<StreamWebSocketControl>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->NoDelay = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ServerCredentialGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocketControl *wrapper = StreamWebSocketControl::Unwrap<StreamWebSocketControl>(info.This());

      try 
      {
        ::Windows::Security::Credentials::PasswordCredential^ result = wrapper->_instance->ServerCredential;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Security.Credentials", "PasswordCredential", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ServerCredentialSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Security::Credentials::PasswordCredential^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketControl^>(info.This()))
      {
        return;
      }

      StreamWebSocketControl *wrapper = StreamWebSocketControl::Unwrap<StreamWebSocketControl>(info.This());

      try 
      {
        
        ::Windows::Security::Credentials::PasswordCredential^ winRtValue = dynamic_cast<::Windows::Security::Credentials::PasswordCredential^>(NodeRT::Utils::GetObjectInstance(value));

        wrapper->_instance->ServerCredential = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ProxyCredentialGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocketControl *wrapper = StreamWebSocketControl::Unwrap<StreamWebSocketControl>(info.This());

      try 
      {
        ::Windows::Security::Credentials::PasswordCredential^ result = wrapper->_instance->ProxyCredential;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Security.Credentials", "PasswordCredential", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ProxyCredentialSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Security::Credentials::PasswordCredential^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketControl^>(info.This()))
      {
        return;
      }

      StreamWebSocketControl *wrapper = StreamWebSocketControl::Unwrap<StreamWebSocketControl>(info.This());

      try 
      {
        
        ::Windows::Security::Credentials::PasswordCredential^ winRtValue = dynamic_cast<::Windows::Security::Credentials::PasswordCredential^>(NodeRT::Utils::GetObjectInstance(value));

        wrapper->_instance->ProxyCredential = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> OutboundBufferSizeInBytesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocketControl *wrapper = StreamWebSocketControl::Unwrap<StreamWebSocketControl>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->OutboundBufferSizeInBytes;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void OutboundBufferSizeInBytesSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketControl^>(info.This()))
      {
        return;
      }

      StreamWebSocketControl *wrapper = StreamWebSocketControl::Unwrap<StreamWebSocketControl>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->OutboundBufferSizeInBytes = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> SupportedProtocolsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketControl^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocketControl *wrapper = StreamWebSocketControl::Unwrap<StreamWebSocketControl>(info.This());

      try 
      {
        ::Windows::Foundation::Collections::IVector<::Platform::String^>^ result = wrapper->_instance->SupportedProtocols;
        return scope.Close(NodeRT::Collections::VectorWrapper<::Platform::String^>::CreateVectorWrapper(result, 
            [](::Platform::String^ val) -> Handle<Value> {
              return NodeRT::Utils::NewString(val->Data());
            },
            [](Handle<Value> value) -> bool {
              return value->IsString();
            },
            [](Handle<Value> value) -> ::Platform::String^ {
              return ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));
            }
          ));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::StreamWebSocketControl^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapStreamWebSocketControl(::Windows::Networking::Sockets::StreamWebSocketControl^ wintRtInstance);
    friend ::Windows::Networking::Sockets::StreamWebSocketControl^ UnwrapStreamWebSocketControl(Handle<Value> value);
    friend bool IsStreamWebSocketControlWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> StreamWebSocketControl::s_constructorTemplate;

  v8::Handle<v8::Value> WrapStreamWebSocketControl(::Windows::Networking::Sockets::StreamWebSocketControl^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(StreamWebSocketControl::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::StreamWebSocketControl^ UnwrapStreamWebSocketControl(Handle<Value> value)
  {
     return StreamWebSocketControl::Unwrap<StreamWebSocketControl>(value.As<Object>())->_instance;
  }

  void InitStreamWebSocketControl(Handle<Object> exports)
  {
    StreamWebSocketControl::Init(exports);
  }

  class StreamWebSocketInformation : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("StreamWebSocketInformation"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("bandwidthStatistics"), BandwidthStatisticsGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("localAddress"), LocalAddressGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("protocol"), ProtocolGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("StreamWebSocketInformation"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    StreamWebSocketInformation(::Windows::Networking::Sockets::StreamWebSocketInformation^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::StreamWebSocketInformation^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketInformation^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::StreamWebSocketInformation^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      StreamWebSocketInformation *wrapperInstance = new StreamWebSocketInformation(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> BandwidthStatisticsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocketInformation *wrapper = StreamWebSocketInformation::Unwrap<StreamWebSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::BandwidthStatistics result = wrapper->_instance->BandwidthStatistics;
        return scope.Close(BandwidthStatisticsToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> LocalAddressGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocketInformation *wrapper = StreamWebSocketInformation::Unwrap<StreamWebSocketInformation>(info.This());

      try 
      {
        ::Windows::Networking::HostName^ result = wrapper->_instance->LocalAddress;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Networking", "HostName", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ProtocolGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocketInformation^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocketInformation *wrapper = StreamWebSocketInformation::Unwrap<StreamWebSocketInformation>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Protocol;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Networking::Sockets::StreamWebSocketInformation^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapStreamWebSocketInformation(::Windows::Networking::Sockets::StreamWebSocketInformation^ wintRtInstance);
    friend ::Windows::Networking::Sockets::StreamWebSocketInformation^ UnwrapStreamWebSocketInformation(Handle<Value> value);
    friend bool IsStreamWebSocketInformationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> StreamWebSocketInformation::s_constructorTemplate;

  v8::Handle<v8::Value> WrapStreamWebSocketInformation(::Windows::Networking::Sockets::StreamWebSocketInformation^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(StreamWebSocketInformation::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::StreamWebSocketInformation^ UnwrapStreamWebSocketInformation(Handle<Value> value)
  {
     return StreamWebSocketInformation::Unwrap<StreamWebSocketInformation>(value.As<Object>())->_instance;
  }

  void InitStreamWebSocketInformation(Handle<Object> exports)
  {
    StreamWebSocketInformation::Init(exports);
  }

  class StreamWebSocket : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("StreamWebSocket"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("setRequestHeader"), FunctionTemplate::New(SetRequestHeader)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("close"), FunctionTemplate::New(Close)->GetFunction());
      
            
      func = FunctionTemplate::New(ConnectAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("connectAsync"), func);
      
            
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("control"), ControlGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("information"), InformationGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("inputStream"), InputStreamGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("outputStream"), OutputStreamGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("StreamWebSocket"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    StreamWebSocket(::Windows::Networking::Sockets::StreamWebSocket^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::StreamWebSocket^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocket^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::StreamWebSocket^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Networking::Sockets::StreamWebSocket();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      StreamWebSocket *wrapperInstance = new StreamWebSocket(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> ConnectAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      StreamWebSocket *wrapper = StreamWebSocket::Unwrap<StreamWebSocket>(args.This());

      ::Windows::Foundation::IAsyncAction^ op;
    

      if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Uri^>(args[0]))
      {
        try
        {
          ::Windows::Foundation::Uri^ arg0 = dynamic_cast<::Windows::Foundation::Uri^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          op = wrapper->_instance->ConnectAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<void> t) 
      {	
        try
        {
          t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            Handle<Value> args[] = {Undefined()};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> SetRequestHeader(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocket *wrapper = StreamWebSocket::Unwrap<StreamWebSocket>(args.This());

      if (args.Length() == 2
        && args[0]->IsString()
        && args[1]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          Platform::String^ arg1 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[1])));
          
          wrapper->_instance->SetRequestHeader(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Close(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocket^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocket *wrapper = StreamWebSocket::Unwrap<StreamWebSocket>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          delete wrapper->_instance;
          wrapper->_instance = nullptr;
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }




    static Handle<Value> ControlGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocket *wrapper = StreamWebSocket::Unwrap<StreamWebSocket>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::StreamWebSocketControl^ result = wrapper->_instance->Control;
        return scope.Close(WrapStreamWebSocketControl(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> InformationGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocket *wrapper = StreamWebSocket::Unwrap<StreamWebSocket>(info.This());

      try 
      {
        ::Windows::Networking::Sockets::StreamWebSocketInformation^ result = wrapper->_instance->Information;
        return scope.Close(WrapStreamWebSocketInformation(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> InputStreamGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocket *wrapper = StreamWebSocket::Unwrap<StreamWebSocket>(info.This());

      try 
      {
        ::Windows::Storage::Streams::IInputStream^ result = wrapper->_instance->InputStream;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IInputStream", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> OutputStreamGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocket^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      StreamWebSocket *wrapper = StreamWebSocket::Unwrap<StreamWebSocket>(info.This());

      try 
      {
        ::Windows::Storage::Streams::IOutputStream^ result = wrapper->_instance->OutputStream;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage.Streams", "IOutputStream", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"closed", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocket^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        StreamWebSocket *wrapper = StreamWebSocket::Unwrap<StreamWebSocket>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->Closed::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::Networking::Sockets::IWebSocket^, ::Windows::Networking::Sockets::WebSocketClosedEventArgs^>(
            [callbackObjPtr](::Windows::Networking::Sockets::IWebSocket^ arg0, ::Windows::Networking::Sockets::WebSocketClosedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapIWebSocket(arg0);
                Handle<Value> wrappedArg1 = WrapWebSocketClosedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"closed", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"closed", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::StreamWebSocket^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          StreamWebSocket *wrapper = StreamWebSocket::Unwrap<StreamWebSocket>(args.This());
          wrapper->_instance->Closed::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::Networking::Sockets::StreamWebSocket^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapStreamWebSocket(::Windows::Networking::Sockets::StreamWebSocket^ wintRtInstance);
    friend ::Windows::Networking::Sockets::StreamWebSocket^ UnwrapStreamWebSocket(Handle<Value> value);
    friend bool IsStreamWebSocketWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> StreamWebSocket::s_constructorTemplate;

  v8::Handle<v8::Value> WrapStreamWebSocket(::Windows::Networking::Sockets::StreamWebSocket^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(StreamWebSocket::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::StreamWebSocket^ UnwrapStreamWebSocket(Handle<Value> value)
  {
     return StreamWebSocket::Unwrap<StreamWebSocket>(value.As<Object>())->_instance;
  }

  void InitStreamWebSocket(Handle<Object> exports)
  {
    StreamWebSocket::Init(exports);
  }

  class WebSocketKeepAlive : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("WebSocketKeepAlive"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("run"), FunctionTemplate::New(Run)->GetFunction());
      
                        
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("WebSocketKeepAlive"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    WebSocketKeepAlive(::Windows::Networking::Sockets::WebSocketKeepAlive^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::WebSocketKeepAlive^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::WebSocketKeepAlive^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::WebSocketKeepAlive^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Networking::Sockets::WebSocketKeepAlive();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      WebSocketKeepAlive *wrapperInstance = new WebSocketKeepAlive(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> Run(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::WebSocketKeepAlive^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      WebSocketKeepAlive *wrapper = WebSocketKeepAlive::Unwrap<WebSocketKeepAlive>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::ApplicationModel::Background::IBackgroundTaskInstance^>(args[0]))
      {
        try
        {
          ::Windows::ApplicationModel::Background::IBackgroundTaskInstance^ arg0 = dynamic_cast<::Windows::ApplicationModel::Background::IBackgroundTaskInstance^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          wrapper->_instance->Run(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }





  private:
    ::Windows::Networking::Sockets::WebSocketKeepAlive^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapWebSocketKeepAlive(::Windows::Networking::Sockets::WebSocketKeepAlive^ wintRtInstance);
    friend ::Windows::Networking::Sockets::WebSocketKeepAlive^ UnwrapWebSocketKeepAlive(Handle<Value> value);
    friend bool IsWebSocketKeepAliveWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> WebSocketKeepAlive::s_constructorTemplate;

  v8::Handle<v8::Value> WrapWebSocketKeepAlive(::Windows::Networking::Sockets::WebSocketKeepAlive^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(WebSocketKeepAlive::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::WebSocketKeepAlive^ UnwrapWebSocketKeepAlive(Handle<Value> value)
  {
     return WebSocketKeepAlive::Unwrap<WebSocketKeepAlive>(value.As<Object>())->_instance;
  }

  void InitWebSocketKeepAlive(Handle<Object> exports)
  {
    WebSocketKeepAlive::Init(exports);
  }

  class SocketError : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("SocketError"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("getStatus"), FunctionTemplate::New(GetStatus)->GetFunction());

      exports->Set(String::NewSymbol("SocketError"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    SocketError(::Windows::Networking::Sockets::SocketError^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::SocketError^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::SocketError^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::SocketError^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      SocketError *wrapperInstance = new SocketError(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  


    static Handle<Value> GetStatus(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          int arg0 = static_cast<int>(args[0]->Int32Value());
          
          ::Windows::Networking::Sockets::SocketErrorStatus result;
          result = ::Windows::Networking::Sockets::SocketError::GetStatus(arg0);
          return scope.Close(Integer::New(static_cast<int>(result)));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



  private:
    ::Windows::Networking::Sockets::SocketError^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapSocketError(::Windows::Networking::Sockets::SocketError^ wintRtInstance);
    friend ::Windows::Networking::Sockets::SocketError^ UnwrapSocketError(Handle<Value> value);
    friend bool IsSocketErrorWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> SocketError::s_constructorTemplate;

  v8::Handle<v8::Value> WrapSocketError(::Windows::Networking::Sockets::SocketError^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(SocketError::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::SocketError^ UnwrapSocketError(Handle<Value> value)
  {
     return SocketError::Unwrap<SocketError>(value.As<Object>())->_instance;
  }

  void InitSocketError(Handle<Object> exports)
  {
    SocketError::Init(exports);
  }

  class WebSocketError : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("WebSocketError"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("getStatus"), FunctionTemplate::New(GetStatus)->GetFunction());

      exports->Set(String::NewSymbol("WebSocketError"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    WebSocketError(::Windows::Networking::Sockets::WebSocketError^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Networking::Sockets::WebSocketError^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Networking::Sockets::WebSocketError^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Networking::Sockets::WebSocketError^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      WebSocketError *wrapperInstance = new WebSocketError(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  


    static Handle<Value> GetStatus(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          int arg0 = static_cast<int>(args[0]->Int32Value());
          
          ::Windows::Web::WebErrorStatus result;
          result = ::Windows::Networking::Sockets::WebSocketError::GetStatus(arg0);
          return scope.Close(Integer::New(static_cast<int>(result)));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



  private:
    ::Windows::Networking::Sockets::WebSocketError^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapWebSocketError(::Windows::Networking::Sockets::WebSocketError^ wintRtInstance);
    friend ::Windows::Networking::Sockets::WebSocketError^ UnwrapWebSocketError(Handle<Value> value);
    friend bool IsWebSocketErrorWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> WebSocketError::s_constructorTemplate;

  v8::Handle<v8::Value> WrapWebSocketError(::Windows::Networking::Sockets::WebSocketError^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(WebSocketError::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Networking::Sockets::WebSocketError^ UnwrapWebSocketError(Handle<Value> value)
  {
     return WebSocketError::Unwrap<WebSocketError>(value.As<Object>())->_instance;
  }

  void InitWebSocketError(Handle<Object> exports)
  {
    WebSocketError::Init(exports);
  }

} } } } 

void init(Handle<Object> exports)
{
  if (FAILED(CoInitializeEx(nullptr, COINIT_MULTITHREADED)))
  {
    ThrowException(v8::Exception::Error(NodeRT::Utils::NewString(L"error in CoInitializeEx()")));
    return;
  }
  
  NodeRT::Windows::Networking::Sockets::InitControlChannelTriggerStatusEnum(exports);
  NodeRT::Windows::Networking::Sockets::InitControlChannelTriggerResourceTypeEnum(exports);
  NodeRT::Windows::Networking::Sockets::InitControlChannelTriggerResetReasonEnum(exports);
  NodeRT::Windows::Networking::Sockets::InitSocketMessageTypeEnum(exports);
  NodeRT::Windows::Networking::Sockets::InitSocketProtectionLevelEnum(exports);
  NodeRT::Windows::Networking::Sockets::InitSocketQualityOfServiceEnum(exports);
  NodeRT::Windows::Networking::Sockets::InitSocketErrorStatusEnum(exports);
  NodeRT::Windows::Networking::Sockets::InitSocketSslErrorSeverityEnum(exports);
  NodeRT::Windows::Networking::Sockets::InitControlChannelTrigger(exports);
  NodeRT::Windows::Networking::Sockets::InitIControlChannelTriggerEventDetails(exports);
  NodeRT::Windows::Networking::Sockets::InitIControlChannelTriggerResetEventDetails(exports);
  NodeRT::Windows::Networking::Sockets::InitDatagramSocketControl(exports);
  NodeRT::Windows::Networking::Sockets::InitDatagramSocketInformation(exports);
  NodeRT::Windows::Networking::Sockets::InitDatagramSocket(exports);
  NodeRT::Windows::Networking::Sockets::InitDatagramSocketMessageReceivedEventArgs(exports);
  NodeRT::Windows::Networking::Sockets::InitStreamSocketControl(exports);
  NodeRT::Windows::Networking::Sockets::InitStreamSocketInformation(exports);
  NodeRT::Windows::Networking::Sockets::InitStreamSocket(exports);
  NodeRT::Windows::Networking::Sockets::InitStreamSocketListenerControl(exports);
  NodeRT::Windows::Networking::Sockets::InitStreamSocketListenerInformation(exports);
  NodeRT::Windows::Networking::Sockets::InitStreamSocketListener(exports);
  NodeRT::Windows::Networking::Sockets::InitStreamSocketListenerConnectionReceivedEventArgs(exports);
  NodeRT::Windows::Networking::Sockets::InitIWebSocketControl(exports);
  NodeRT::Windows::Networking::Sockets::InitIWebSocketInformation(exports);
  NodeRT::Windows::Networking::Sockets::InitIWebSocket(exports);
  NodeRT::Windows::Networking::Sockets::InitWebSocketClosedEventArgs(exports);
  NodeRT::Windows::Networking::Sockets::InitMessageWebSocketControl(exports);
  NodeRT::Windows::Networking::Sockets::InitMessageWebSocketInformation(exports);
  NodeRT::Windows::Networking::Sockets::InitMessageWebSocket(exports);
  NodeRT::Windows::Networking::Sockets::InitMessageWebSocketMessageReceivedEventArgs(exports);
  NodeRT::Windows::Networking::Sockets::InitStreamWebSocketControl(exports);
  NodeRT::Windows::Networking::Sockets::InitStreamWebSocketInformation(exports);
  NodeRT::Windows::Networking::Sockets::InitStreamWebSocket(exports);
  NodeRT::Windows::Networking::Sockets::InitWebSocketKeepAlive(exports);
  NodeRT::Windows::Networking::Sockets::InitSocketError(exports);
  NodeRT::Windows::Networking::Sockets::InitWebSocketError(exports);

  NodeRT::Utils::RegisterNameSpace("Windows.Networking.Sockets", exports);
}


NODE_MODULE(binding, init)